function spl = FactorAnalysis(X,c0,d0,g0,h0,burnin,num,space)

% X should be normalized to zero mean for each row. This is done prior to
% calling this function (see the main code for more details).

randn('state',0); rand('state',0);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Model input data:
% X is the data matrix with dimension p*n (n samples, p genes).
% c0 and d0 are Gamma hyperparameters for the precision of additive noise (p*1)
% g0 and h0 are Gamma hyperparameters for the precision of factor loading (p*k)


% Model output:
% spl.A contains samples of factor loading (matrix A in the model).
% spl.S contains samples of factors (matrix S in the model).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rand('state',0); randn('state',0); 

[p,n] = size(X); k = size(g0,2); % here, 'p' represents the number of images under investigation and 'n' the number of features. 'k' is the number of factors.

% Initialization for factors.
S.mu = zeros(k,n); 
S.Sig = eye(k);
S.spl = zeros(k,n);

% Initialization for precision of additive noise.
phi.c = c0;
phi.d = d0;
phi.spl = ones(size(c0));

% Initialization for factor loading.
A.spl =  [eye(k);zeros(p-k,k)];
% A.spl = A0;
% A.spl = randn(p,k+1);

% Initialization for precision of factor loading.
alpha.g = g0;
alpha.h = h0;
alpha.spl = ones(size(g0));
spl.S = zeros(k,n);
spl.phi = zeros(p,1);
spl.A = zeros(p,k);
spl.alpha = zeros(p,k);
maxit = burnin + num*space; % num is the number of Gibbs samplings; burnin is the number of burn-in iterations (for more details, please read the paper).
iter = 0; % Gibbs iterations:

while (iter<maxit) 
    iter = iter + 1;
    C = A.spl'.*repmat((phi.spl)',[k,1]);
    S.Sig = inv(eye(k)+C*A.spl);
    S.mu = S.Sig*(C*X);
    for i = 1:n
        S.spl(:,i) = rMNorm(S.mu(:,i),S.Sig,1);
    end
    S2 = S.spl*S.spl'; XS = X*S.spl';
    for j = 1:p
        G = chol(phi.spl(j)*S2+diag(alpha.spl(j,:)));
        A.spl(j,:) = G\(randn(k,1)+phi.spl(j)*((G')\(XS(j,:)')));
    end

    alpha.g = g0 + 0.5;
    alpha.h = h0 + 0.5*A.spl.^2; 
    alpha.spl = gamrnd(alpha.g,1./alpha.h);
    
    res = X - A.spl*S.spl;
    phi.c = c0 + 0.5*n;
    phi.d = d0 + 0.5*sum(res.^2,2);
    phi.spl = gamrnd(phi.c,1./phi.d);
    
    ndx = iter - burnin;
    test = mod(ndx,space);
    if (ndx > 0) && (test == 0)
        spl.S = spl.S + S.spl/num;
        spl.phi = spl.phi + phi.spl/num;
        spl.A = spl.A + A.spl/num;
        spl.alpha = spl.alpha + alpha.spl/num;
    end
    disp(['Current iteration: ' num2str(iter)]);
end