package edu.duke.cs.smlr;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Iterator;
import java.util.TreeSet;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import com.apple.eawt.Application;
import com.apple.eawt.ApplicationAdapter;

import edu.duke.cs.smlr.gui.MacAppListener;
import edu.duke.cs.smlr.gui.MenuMaker;
import edu.duke.cs.smlr.gui.frames.PreferencesFrame;
import edu.duke.cs.smlr.gui.frames.AboutFrame;
import edu.duke.cs.smlr.gui.panels.ChoosePanel;
import edu.duke.cs.smlr.gui.panels.DataPanel;
import edu.duke.cs.smlr.gui.panels.DataPanelTop;
import edu.duke.cs.smlr.gui.panels.MethodsPanel;
import edu.duke.cs.smlr.gui.panels.TaskPanel;
import edu.duke.cs.smlr.learn.properties.DisplayProperty;
import edu.duke.cs.smlr.learn.properties.LearnProperty;
import edu.duke.cs.smlr.learn.properties.ModifyProperty;
import edu.duke.cs.smlr.util.Constants;
import edu.duke.cs.smlr.util.Core;
import edu.duke.cs.smlr.util.Defaults;
import edu.duke.cs.smlr.util.HaltingException;
import edu.duke.cs.smlr.util.Messages;
import edu.duke.cs.smlr.util.SwingWorker;

/**
 * Main class for the GUI, used to create a settings file or perform other tasks
 * related to machine learning
 * 
 * @author Matt Edwards, Jason Bosko
 */
public class Graphical extends JFrame implements ComponentListener {
	private TaskPanel taskPanel;
	private DataPanel dataTrainPanel;
	private DataPanel dataTestPanel;
	private DataPanel dataUnlabeledPanel;
	private DataPanelTop dataTop;
	private MethodsPanel methodsPanel;
	private ChoosePanel choosePanel;
	private JTabbedPane tabbedPane;
	private PreferencesFrame preferencesFrame;
	private AboutFrame aboutFrame;
	
	/**Ugly, public flag for "Load" to tell "Data" to reload
	 */
	public boolean newData = false;
	public boolean newTestData = false;
	public boolean newUnlabeledData = false;
	public boolean isMacOSX = false;
	
	private LearnProperty settingsProperty[];
	
	ModifyProperty modPropTrain;
	ModifyProperty modPropTest;
	ModifyProperty modPropUnlabeled;
	
	/**
	 * Function used to take an array and copy it <b>x</b> times. For example, 
	 * the array [a,b,c] would get expanded to [a,a,a,b,b,b,c,c,c] if <b>x</b>
	 * were 3.
	 * @param curr The original array
	 * @param x The factor of expansion
	 * @return The expanded array.
	 */
	private LearnProperty[] expand(LearnProperty[] curr, int x) {
		if (x <= 1)
			return curr;
		LearnProperty[] temp = new LearnProperty[curr.length * x];
		for (int i = 0; i < x; ++i)
			for (int j = 0; j < curr.length; ++j)
				temp[i * curr.length + j] = new LearnProperty(curr[j]);
		return temp;
	}
	
	/**
	 * Function used to take an array, <b>replacement</b>, and stick it in another array, 
	 * <b>curr</b>, at position <b>index</b>. For example, if <b>curr</b> is [a,b,c] 
	 * and <b>index</b> is 1 and <b>replacement</b> is [d,e], then the answer would be 
	 * the array [a,d,e,c].
	 * @param curr The original array
	 * @param replacement The array of things to put into <b>curr</b>
	 * @param index The index at which to modify <b>curr</b>
	 * @return The new array
	 */
	private LearnProperty[] replaceOneWithMany(LearnProperty[] curr, LearnProperty replacement[], int index) {
		int x = replacement.length;
		LearnProperty temp[] = new LearnProperty[curr.length + x - 1];
		int tempIndex = 0;
		for (int i = 0; i < curr.length; ++i) {
			if (i == index) {
				for (int k = 0; k < x; ++k)
					temp[i+k] = new LearnProperty(replacement[k]);
				tempIndex += x;
			}
			else {
				temp[tempIndex] = new LearnProperty(curr[i]);
				tempIndex++;
			}
		}
		return temp;
	}
	
	/**
	 * Constructs the Graphical user interface
	 *
	 */
	public Graphical() {
		try {
			if(System.getProperty("os.name").indexOf("Mac") != -1) {
				isMacOSX = true;
			} 
		} catch (SecurityException e) { }

		if(isMacOSX) { 
			//we can't call these commands explicitly because in non-macs their compilation will fail (importing stuff for macAppListener)
//			com.apple.eawt.Application macApp = new com.apple.eawt.Application();
//			macApp.setEnabledPreferencesMenu(true);			
//			macApp.addApplicationListener(new MacAppListener(this));
			
			Class [] oneBooleanClass = {boolean.class};
			Object [] oneBooleanTrue = {Boolean.TRUE};	
			Class [] oneGraphicalClass = {Graphical.class};
			Object [] oneGraphicalThis = {this};
			//scary reflection because these classes aren't found on non-apples, so we can't
			//directly reference them in code that runs on non-apples
			try {
				Class macAppClass = Class.forName("com.apple.eawt.Application");
				Object macApp = macAppClass.newInstance();
				com.apple.eawt.Application realMacApp = new com.apple.eawt.Application();//
				//realMacApp.add;
				Method method1 = macAppClass.getMethod("setEnabledPreferencesMenu", oneBooleanClass);
				method1.invoke(macApp, oneBooleanTrue);
				//macApp.setEnabledPreferencesMenu(true);
				Class theirMacAppListenerClass = Class.forName("com.apple.eawt.ApplicationListener");
				Class ourMacAppListenerClass = Class.forName("edu.duke.cs.smlr.gui.MacAppListener"); 
				Class [] oneListenerClass = {theirMacAppListenerClass};
				Method method2 = macAppClass.getMethod("addApplicationListener", oneListenerClass);				
				Constructor listenerConstructor = ourMacAppListenerClass.getConstructor(oneGraphicalClass);
				Object [] newListener = {listenerConstructor.newInstance(oneGraphicalThis)};
				method2.invoke(macApp, newListener);
				//macApp.addApplicationListener(new MacAppListener(this));
			} catch (ClassNotFoundException e) {
				e.printStackTrace();
			} catch (InstantiationException e) {
				e.printStackTrace();
				ApplicationAdapter aa = new ApplicationAdapter();
				
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (SecurityException e) {
				e.printStackTrace();
				Application bob = new Application();
				bob.setEnabledPreferencesMenu(true);
			} catch (NoSuchMethodException e) {
				e.printStackTrace();
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			}        
		}
		
		setTitle(Messages.TITLE_MAIN);
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		MenuMaker.makeMenus(this);
		this.setSize(650, 500);
		
		JPanel topPanel = new JPanel();
		topPanel.setLayout(new BorderLayout());
		getContentPane().add(topPanel);
		
		setPanels();
		dataTop.addTab(dataTrainPanel, Messages.TITLE_DATA_TRAIN, Messages.TOOLTIP_DATA_TRAIN);
		dataTop.addTab(dataTestPanel, Messages.TITLE_DATA_TEST, Messages.TOOLTIP_DATA_TEST);
		dataTop.addTab(dataUnlabeledPanel, Messages.TITLE_DATA_UNLABELED, Messages.TOOLTIP_DATA_UNLABELED);
		tabbedPane = new JTabbedPane();
		tabbedPane.addTab(Messages.TITLE_TASK, null, taskPanel.getPanel(), Messages.TOOLTIP_TASK);
		tabbedPane.addTab(Messages.TITLE_DATA, null, dataTop.getPanel(), Messages.TOOLTIP_DATA);
		tabbedPane.addTab(Messages.TITLE_METHODS, null, methodsPanel.getPanel(), Messages.TOOLTIP_METHODS);
		tabbedPane.addTab(Messages.TITLE_CHOOSE, null, choosePanel.getPanel(), Messages.TOOLTIP_CHOOSE);
		
		choosePanel.getPanel().addComponentListener(this);
		dataTop.getPanel().addComponentListener(this);
		dataTrainPanel.getPanel().addComponentListener(this);
		dataTestPanel.getPanel().addComponentListener(this);
		dataUnlabeledPanel.getPanel().addComponentListener(this);
		
		topPanel.add(tabbedPane, BorderLayout.CENTER);
		
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		this.setLocation(
				(dim.width - this.getWidth()) / 2,
				(dim.height - this.getHeight()) / 2);
		
		this.setVisible(true);
	}
	
	public void setPanels() {
		taskPanel = new TaskPanel(this);
		dataTrainPanel = new DataPanel(this);
		dataTestPanel = new DataPanel(this);
		dataUnlabeledPanel = new DataPanel(this);
		dataTop = new DataPanelTop(this);
		methodsPanel = new MethodsPanel(this);
		choosePanel = new ChoosePanel(this);
		preferencesFrame = new PreferencesFrame(this);
		aboutFrame = new AboutFrame(this);
	}
	
	/**
	 * Makes the "About..." frame visible
	 *
	 */
	public void showAboutFrame() {
		aboutFrame.setVisible(true);
	}
	
	/**
	 * Makes the preferences frame visible
	 *
	 */
	public void showPreferencesFrame() {
		preferencesFrame.setVisible(true);
	}
	
	/**
	 * Automatically sets the current panel to be the task panel
	 *
	 */
	public void switchToTaskPanel() {
		tabbedPane.setSelectedComponent(taskPanel.getPanel());
	}
	
	public void componentResized(ComponentEvent e) { }
	
	public void componentMoved(ComponentEvent e) { }
	
	public void componentHidden(ComponentEvent e) {
		if(e.getSource() == choosePanel.getPanel()) {
			choosePanel.updateUnselected();
		}
	}
	
	public void componentShown(ComponentEvent e) {
		// Either the data panel or the choose panel has been selected
		if(e.getSource() == dataTrainPanel.getPanel() || (e.getSource() == dataTop.getPanel() && dataTrainPanel.getPanel().isShowing())) { 
			// Check to see if the data panel needs to load data
			if(taskPanel.isTrainingLoaded()) {
				if(newData) {
					dataTrainPanel.showLoadingPanel();
					SwingWorker worker = new SwingWorker() {
						public Object construct() {
							try {
								dataTrainPanel.loadTable(taskPanel.getTrainingFile(), taskPanel.getClassColumn(), modPropTrain);
							} catch (HaltingException e1b) {
								JOptionPane.showMessageDialog(dataTrainPanel.getPanel(),
										Messages.ERROR_FILE_FORMAT,
										Messages.ERROR_FILE_FORMAT_TITLE, JOptionPane.ERROR_MESSAGE);
								dataTrainPanel.showUnloadedPanel();
								return "Not loaded";
							}
							newData = false;
							return "Loaded.";
						}
						public void finished() {
							if(this.get().equals("Loaded."))
								dataTrainPanel.showLoadedPanel();
						}
					};
					worker.start();
				}
			}
			else {
				dataTrainPanel.showUnloadedPanel();
			}
		} else if(e.getSource() == dataTestPanel.getPanel() || (e.getSource() == dataTop.getPanel() && dataTestPanel.getPanel().isShowing())) { 
			// Check to see if the data panel needs to load data
			if(taskPanel.isTestLoaded()) {
				if(newTestData) {
					dataTestPanel.showLoadingPanel();
					SwingWorker worker = new SwingWorker() {
						public Object construct() {
							try {
								dataTestPanel.loadTable(taskPanel.getTestFile(), taskPanel.getClassColumn(), modPropTest);
							} catch (HaltingException e1b) {
								JOptionPane.showMessageDialog(dataTestPanel.getPanel(),
										Messages.ERROR_FILE_FORMAT,
										Messages.ERROR_FILE_FORMAT_TITLE, JOptionPane.ERROR_MESSAGE);
								dataTestPanel.showUnloadedPanel();
								return "Not loaded";
							}
							newTestData = false;
							return "Loaded.";
						}
						public void finished() {
							if(this.get().equals("Loaded."))
								dataTestPanel.showLoadedPanel();
						}
					};
					worker.start();
				}
			}
			else {
				dataTestPanel.showUnloadedPanel();
			}
		} if(e.getSource() == dataUnlabeledPanel.getPanel() || (e.getSource() == dataTop.getPanel() && dataUnlabeledPanel.getPanel().isShowing())) { 
			// Check to see if the data panel needs to load data
			if(taskPanel.isUnlabeledLoaded()) {
				if(newUnlabeledData) {
					dataUnlabeledPanel.showLoadingPanel();
					SwingWorker worker = new SwingWorker() {
						public Object construct() {
							try {
								dataUnlabeledPanel.loadTable(taskPanel.getUnlabeledFile(), Constants.READER_UNCLASSIFIED, modPropUnlabeled);
							} catch (HaltingException e1b) {
								JOptionPane.showMessageDialog(dataUnlabeledPanel.getPanel(),
										Messages.ERROR_FILE_FORMAT,
										Messages.ERROR_FILE_FORMAT_TITLE, JOptionPane.ERROR_MESSAGE);
								dataUnlabeledPanel.showUnloadedPanel();
								return "Not loaded";
							}
							newUnlabeledData = false;
							return "Loaded.";
						}
						public void finished() {
							if(this.get().equals("Loaded."))
								dataUnlabeledPanel.showLoadedPanel();
						}
					};
					worker.start();
				}
			}
			else {
				dataUnlabeledPanel.showUnloadedPanel();
			}
		}
		
		if(e.getSource() == choosePanel.getPanel()) {
			// update the selections in the choose panel
			makeSettingsProperty();
			choosePanel.update(settingsProperty);
		}
	}
	
	/**
	 * Asks the task panel if a test file has been given
	 * @return Whether a test file has been given
	 */
	public boolean isTestLoaded() {
		return taskPanel.isTestLoaded();
	}
	
	/**
	 * Asks the task panel if a training file has been given
	 * @return Whether a training file has been given
	 */
	public boolean isTrainingLoaded() {
		return taskPanel.isTrainingLoaded();
	}
	
	/**
	 * Asks the task panel if an unlabeled file has been given
	 * @return Whether an unlabeled file has been given
	 */
	public boolean isUnlabeledLoaded() {
		return taskPanel.isUnlabeledLoaded();
	}
	
	/**
	 * Asks the task panel for the training file
	 * @return The training file
	 */
	public String getTrainingFileName() {
		return taskPanel.getTrainingFile();
	}
	
	/**
	 * Asks the task panel for the column containing the class labels
	 * @return The column containing the class labels
	 */
	public int getClassColumn() {
		return taskPanel.getClassColumn();
	}
	
	/**
	 * Asks the task panel if k-fold cross-validation was selected as the behavior
	 * @return Whether k-fold cross-validation was selected as the behavior
	 */
	public boolean isKFoldSelected() {
		return taskPanel.getTestChoice() == Constants.TEST_KFOLD;
	}
	
	/**
	 * Asks the task panel for the type of testing that has been requested. 
	 * This can be test on test data, k-fold CV, or LOOCV.
	 * @return The type of testing that has been requestion
	 */
	public int getEvalChoice() {
		return taskPanel.getTestChoice();
	}
	
	/**
	 * Tells the preferences frame to set the visibility of the k-fold display options to <b>on</b>
	 * @param on Whether the dispay should be visible (true) or invisible (false)
	 */
	public void setKFoldDisplay(boolean on) {
		preferencesFrame.getOutputFrame().setKFoldDisplay(on);
	}
	
	/**
	 * Tells the preferences frame to set the visibility of the test display options to <b>on</b>
	 * @param on Whether the dispay should be visible (true) or invisible (false)
	 */
	public void setTestDisplay(boolean on) {
		preferencesFrame.getOutputFrame().setTestDisplay(on);
	}
	
	/**
	 * Tells the choose panel to reenable the option to learn
	 *
	 */
	public void reenableLearn() {
		choosePanel.reenableLearn();		
	}
	
	/**
	 * Forces the set of possible tests to be run to be updated
	 *
	 */
	public void selectTests() {
		makeSettingsProperty();
	}
	
	/**
	 * Queries the individual panels to create a LearnProperty that represents the 
	 * learning task(s) requested
	 *
	 */
	private void makeSettingsProperty() {
		int propCount = 1;
		settingsProperty = new LearnProperty[propCount];
		settingsProperty[0] = new LearnProperty();
		
		// Take care of all the options that are constant across all possible tests
		
		settingsProperty[0].setFileName(taskPanel.getTrainingFile());
		settingsProperty[0].setConvTol(preferencesFrame.getTechnicalFrame().getConvergenceTolerance());
		settingsProperty[0].setCacheExp(preferencesFrame.getTechnicalFrame().isCacheExponentials());
		settingsProperty[0].setKillThreshold(preferencesFrame.getTechnicalFrame().getKillThreshold());
		settingsProperty[0].setMaxIters(preferencesFrame.getTechnicalFrame().getMaximumIterations());
		settingsProperty[0].setUseOverrelaxation(preferencesFrame.getTechnicalFrame().isOverrelaxation());
		settingsProperty[0].setCacheKer(preferencesFrame.getTechnicalFrame().isCacheKernels());
		
		// Take care of all the options for which multiple values can be selected
		
		
		// Normalize can be set to any combination of "true" and "false"
		if (methodsPanel.getNormalizeYes() && methodsPanel.getNormalizeNo()) {
			settingsProperty = expand(settingsProperty, 2);
			for (int k = 0; k < 2; ++k) {
				for (int i = 0; i < propCount; ++i)
					settingsProperty[k * propCount + i].setNormalize(k==0);
			}
			propCount *= 2;
		}
		else if (methodsPanel.getNormalizeNo()) {
			for (int i = 0; i < propCount; ++i)
				settingsProperty[i].setNormalize(false);
		}
		else if (methodsPanel.getNormalizeYes()) {
			for (int i = 0; i < propCount; ++i)
				settingsProperty[i].setNormalize(true);
		}
		else 
			for (int i = 0; i < propCount; ++i)
				settingsProperty[i].setNormalize(Defaults.NORMALIZE);
		
		double biases[] = methodsPanel.getBias();
		int methods[] = methodsPanel.getAlgorithms();
		int kernels[] = methodsPanel.getKernels();
		int priors[] = methodsPanel.getPriors();
		double lambdas[] = methodsPanel.getLambdas();
		
		
//		There are infinately many options for biases
		int bLength = biases.length;
		settingsProperty = expand(settingsProperty, bLength);
		for (int k = 0; k < bLength; k++)
			for (int i = 0; i < propCount; i++)
				settingsProperty[k * propCount + i].setBias(biases[k]);
		
		propCount *= bLength;
		
		// There are 5 different options for kernels
		int kLength = kernels.length;
		settingsProperty = expand(settingsProperty, kLength);
		for (int k = 0; k < kLength; k++) {
			for (int i = 0; i < propCount; i++) {
				settingsProperty[k * propCount + i].setKernel(kernels[k]);
			}
		}
		propCount *= kLength;
		
		//We must scan the newly added kernels for RBF and Polynomial because
		//they can have multiple parameters, and then we must add them, but it is
		//tricky because....it is
		for (int i = 0; i < propCount; i++) {
			if (settingsProperty[i].getKernel() == Constants.KERNEL_POLYNOMIAL) {
				double[] pars = methodsPanel.getPolyPars();
				int pLength = pars.length;
				LearnProperty replacement[] = new LearnProperty[pLength];
				for (int k = 0; k < pLength; ++k) {
					replacement[k] = new LearnProperty(settingsProperty[i]);
					replacement[k].setKernelPar(new Double(pars[k]));
				}
				settingsProperty = replaceOneWithMany(settingsProperty, replacement, i);
				i += pLength - 1;
				propCount += pLength - 1;
			}
			else if (settingsProperty[i].getKernel() == Constants.KERNEL_RBF) {
				double[] pars = methodsPanel.getRBFPars();
				int pLength = pars.length;
				LearnProperty replacement[] = new LearnProperty[pLength];
				for (int k = 0; k < pLength; ++k) {
					replacement[k] = new LearnProperty(settingsProperty[i]);
					replacement[k].setKernelPar(new Double(pars[k]));
				}
				settingsProperty = replaceOneWithMany(settingsProperty, replacement, i);
				i += pLength - 1;
				propCount += pLength - 1;
			}
		}
		
		//There are 2 different options for methods
		int mLength = methods.length;
		settingsProperty = expand(settingsProperty, mLength);
		for (int k = 0; k < mLength; k++)
			for (int i = 0; i < propCount; i++)
				settingsProperty[k * propCount + i].setMethod(methods[k]);
		
		propCount *= mLength;
		
		//There are 3 different options for priors
		int pLength = priors.length;
		settingsProperty = expand(settingsProperty, pLength);
		for (int k = 0; k < pLength; k++)
			for (int i = 0; i < propCount; i++)
				settingsProperty[k * propCount + i].setPrior(priors[k]);
		
		propCount *= pLength;
		
		//There are infinately many options for lambda
		int lLength = lambdas.length;
		settingsProperty = expand(settingsProperty, lLength);
		for (int k = 0; k < lLength; k++)
			for (int i = 0; i < propCount; i++)
				settingsProperty[k * propCount + i].setLambda(lambdas[k]);
		
		propCount *= lLength;
	}
	
	/**
	 * Takes the information out of a learning property, as well as queries individual
	 * panels, to get the settings String that represents one complete task
	 * @param l The learn property
	 * @return A string representing one task
	 */
	public String getSettings(LearnProperty l) {
		String s = "";
		s += "-behavior "
			+ Constants.toString(taskPanel.getTestChoice());
		if (taskPanel.getTestChoice() == Constants.TEST_KFOLD) {
			s += " -kfold " + taskPanel.getK();
			s += " -kfoldbehavior "
				+ Constants.toString(taskPanel.getKfoldOption());
		}
		
		if (taskPanel.isTrainingLoaded())
			s += " -training " + "\"" + taskPanel.getTrainingFile() + "\"";
		if (taskPanel.isTestLoaded())
			s += " -testing " + "\"" + taskPanel.getTestFile() + "\"";
		if (taskPanel.isUnlabeledLoaded())
			s += " -unlabeled " + "\"" + taskPanel.getUnlabeledFile() + "\"";
		
		s += " -saveto " + "\"" + choosePanel.getSaveToFile() + "\"";
		
		s += " -lambda " + l.getLambda();
		s += " -convergencetol " + l.getConvTol();
		s += " -killthreshold " + l.getKillThreshold();
		s += " -maxiterations " + l.getMaxIters();
		s += " -kernel " + Constants.toString(l.getKernel());
		if (l.getKernel() == Constants.KERNEL_POLYNOMIAL
				|| l.getKernel() == Constants.KERNEL_RBF)
			s += " -kernelparam " + l.getKernelPar();
		
		s += " -prior " + Constants.toString(l.getPrior());
		s += " -algorithm " + Constants.toString(l.getMethod());
		
		s += " -overrelaxed " + l.isUseOverrelaxation();
		s += " -cacheexp " + l.isCacheExp();
		s += " -cachekernel " + l.isCacheKer();
		s += " -bias " + (l.getBias() != 0 ? Double.toString(l.getBias()) : "false");
		s += " -normalize " + l.isNormalize();
		
		ModifyProperty mp = dataTrainPanel.getModifyProperty();
		if (!mp.getDeletedRows().isEmpty()) {
			s += " -deleterowstrain";
			Iterator it = mp.getDeletedRows().iterator();
			while (it.hasNext()) {
				s += " " + (((Integer) it.next()).intValue()+1);
			}
		}
		if (!mp.getDeletedColumns().isEmpty()) {
			s += " -deletecolumnstrain";
			Iterator it = mp.getDeletedColumns().iterator();
			while (it.hasNext()) {
				s += " " + (((Integer) it.next()).intValue()+1);
			}
		}
		mp = dataTestPanel.getModifyProperty();
		if (!mp.getDeletedRows().isEmpty()) {
			s += " -deleterowstest";
			Iterator it = mp.getDeletedRows().iterator();
			while (it.hasNext()) {
				s += " " + (((Integer) it.next()).intValue()+1);
			}
		}
		if (!mp.getDeletedColumns().isEmpty()) {
			s += " -deletecolumnstest";
			Iterator it = mp.getDeletedColumns().iterator();
			while (it.hasNext()) {
				s += " " + (((Integer) it.next()).intValue()+1);
			}
		}
		mp = dataUnlabeledPanel.getModifyProperty();
		if (!mp.getDeletedRows().isEmpty()) {
			s += " -deleterowsunlabeled";
			Iterator it = mp.getDeletedRows().iterator();
			while (it.hasNext()) {
				s += " " + (((Integer) it.next()).intValue()+1);
			}
		}
		if (!mp.getDeletedColumns().isEmpty()) {
			s += " -deletecolumnsunlabeled";
			Iterator it = mp.getDeletedColumns().iterator();
			while (it.hasNext()) {
				s += " " + (((Integer) it.next()).intValue()+1);
			}
		}
		
		int temptemp = getClassColumn();
		if (temptemp != Constants.READER_UNCLASSIFIED) {
			s += " -choosecolumn ";        	
			if (temptemp == Constants.READER_FIRST_COLUMN) s+= "first";
			else if (temptemp == Constants.READER_LAST_COLUMN) s+= "last";
			else if (temptemp == Constants.READER_GUESS) s += "guess";
			else s += Integer.toString(temptemp + 1); //command-line parse is one-based
		}       
		
		boolean opts[] = preferencesFrame.getOutputFrame().getDisplayOptions();
		
		s += " -displayweights " + opts[0];
		s += " -displayiterations " + opts[1];
		s += " -displaybasisused " + opts[2];
		s += " -displaytime " + opts[3];
		
		s += " -displayprobabilities " + opts[4]; 
		s += " -displayclassifications " + opts[5];
		s += " -displaynumwrong " + opts[6];
		s += " -displaynumcorrect " + opts[7];
		s += " -displaypercentage " + opts[8];
		s += " -displaymisclassifications " + opts[9];
		
		s += " -displaytotalcorrect " + opts[10];
		s += " -displaytotalwrong " + opts[11];
		s += " -displaytotalpercentage " + opts[12];
		s += " -displayaverageweights " + opts[13];
		s += " -displayweightcounts " + opts[14];
		s += " -displayaverageweightcounts " + opts[15];
		s += " -displayconfusionmatrix " + opts[16];
		s += " -displayroccurve " + opts[17];
		//System.out.println(s);
		return s;
	}	
	
	/**
	 * Saves the current settings of the GUI to <b>fileName</b>. It must know whether or 
	 * not to reload the choose panel because that is where the information will come from
	 * @param fileName The file to save the settings to
	 * @param reloadChoose Whether or not to reload the choose panel
	 */
	public void saveSettings(String fileName, boolean reloadChoose) {
		if (reloadChoose) {
			this.makeSettingsProperty();
			choosePanel.update(this.settingsProperty);
		}
		String settings[] = choosePanel.tasks();
		try {
			PrintWriter writer = new PrintWriter(new FileOutputStream(fileName));
			for (int i = 0; i < settings.length; ++i)
				writer.println(settings[i]);
			writer.close();
			JOptionPane.showMessageDialog(this, Messages.SUCCESS_SAVING_SETTINGS, Messages.SUCCESS_SAVING_SETTINGS_TITLE, JOptionPane.INFORMATION_MESSAGE);
		}
		catch (IOException e) {
			JOptionPane.showMessageDialog(this, Messages.ERROR_SAVING_FILE, Messages.ERROR_SAVING_FILE_TITLE, JOptionPane.ERROR_MESSAGE);
		}
		
	}
	
	/**
	 * Loads the settings from a file into the GUI
	 * 
	 * @param fileName The file with the settings.
	 */
	public void loadSettings(String fileName) {
		//TODO: Make this method use fewer than 100 lines!
		ArrayList sArray = new ArrayList();
		try {
			BufferedReader input = new BufferedReader(new FileReader(fileName));		    
			String s = "";
			while (s != null) {
				s = input.readLine();
				if (s != null)
					sArray.add(s);
			}
		} catch (FileNotFoundException e) {
			//throw new ArgumentFormatException(Messages.ERROR_FILE_NOT_FOUND);
			JOptionPane.showMessageDialog(this, Messages.ERROR_FILE_NOT_FOUND);
			return;
		} catch (IOException e) {
			//throw new ArgumentFormatException(Messages.ERROR_READING_FILE);
			JOptionPane.showMessageDialog(this, Messages.ERROR_FILE_FORMAT);
			return;
		}
		int count = sArray.size();
		
		//Task Panel
		int task = Constants.TEST_LOOCV;
		String trainingFile = "";
		int classChoice = Constants.READER_GUESS;
		String testFile = "";
		String unlabeledFile = "";
		int K = 1;
		int kBehavior = Constants.KFOLD_RATIO_RANDOM;
		
		//Data Panel
		TreeSet rowsToDeleteTrain = new TreeSet();
		TreeSet colsToDeleteTrain = new TreeSet();
		TreeSet rowsToDeleteTest = new TreeSet();
		TreeSet colsToDeleteTest = new TreeSet();
		TreeSet rowsToDeleteUnlabeled = new TreeSet();
		TreeSet colsToDeleteUnlabeled = new TreeSet();
		
		//Methods Panel
		boolean normalizes[] = new boolean[2];
		boolean kernels[] = new boolean[5];
		TreeSet polyParams = new TreeSet();
		TreeSet rbfParams = new TreeSet();
		TreeSet biases = new TreeSet();
		boolean priors[] = new boolean[3];
		TreeSet lambdas = new TreeSet();
		boolean updateRules[] = new boolean[2];
		
		//Output Preferences
		boolean learnDisplays[] = new boolean[4];
		boolean testDisplays[] = new boolean[6];
		boolean crossDisplays[] = new boolean[8];
		String saveFile = "";
		
		//Techinical Preferences
		double killThresh = 1e-4;
		double convTol = 1e-6;
		int maxIter = 10000;
		boolean overrelaxed = false;
		boolean cacheExp = false;
		boolean cacheKer = true;
		
		for (int i = 0; i < count; i++) {
			ArrayList options = new ArrayList(Arrays.asList(((String)sArray.get(i)).split(" ")));
			HashSet optionsSet = new HashSet(options);
			options.add("-[null string]");
			int index = 0;
			String s = "";
			
			if (optionsSet.contains("-behavior")) {
				index = options.indexOf("-behavior") + 1;
				s = (String) options.get(index);
				if (s.equals("testontesting")) task = Constants.TEST_TESTING_DATA;
				if (s.equals("testontraining")) task = Constants.TEST_TRAINING_DATA;
				if (s.equals("kfoldtest")) task = Constants.TEST_KFOLD;
				if (s.equals("loocv")) task = Constants.TEST_LOOCV;
				if (s.equals("unlabeled")) task = Constants.TEST_UNLABELED;
			}
			
			if (optionsSet.contains("-training")) {
				index = options.indexOf("-training") + 1;
				trainingFile = Core.parseFileName(options, index);
				newData = true;
			}
			else trainingFile = Constants.NO_FILE_LOADED;
			
			if (optionsSet.contains("-choosecolumn")) {
				index = options.indexOf("-choosecolumn") + 1;
				String next = (String) options.get(index);
				try {
					int col = Integer.parseInt(next) - 1;
					if (col >= 1) classChoice = col;
					else classChoice = (Constants.READER_GUESS);
				}
				catch (NumberFormatException e) {
					if      (next.equals("first")) classChoice = (Constants.READER_FIRST_COLUMN);
					else if (next.equals("last"))  classChoice = (Constants.READER_LAST_COLUMN);
					else if (next.equals("guess")) classChoice = (Constants.READER_GUESS);
					else						   classChoice = (Constants.READER_GUESS);
				}
			}
			
			if (optionsSet.contains("-testing")) {
				index = options.indexOf("-testing") + 1;
				testFile = Core.parseFileName(options, index);
				newTestData = true;
			}
			
			if (optionsSet.contains("-unlabeled")) {
				index = options.indexOf("-unlabeled") + 1;
				unlabeledFile = Core.parseFileName(options, index);
				newUnlabeledData = true;
			}
			
			if (optionsSet.contains("-kfold")) {
				index = options.indexOf("-kfold") + 1;
				try {
					K = Integer.parseInt((String) options.get(index));
				} catch (NumberFormatException e) {
					K = 1;
				}
			}
			else K = 1;
			
			if (optionsSet.contains("-kfoldbehavior")) {
				index = options.indexOf("-kfoldbehavior") + 1;
				s = (String) options.get(index); 
				if (s.equals("ratiorandom")) kBehavior = Constants.KFOLD_RATIO_RANDOM;
				if (s.equals("random"))      kBehavior = Constants.KFOLD_PURE_RANDOM;
				if (s.equals("inplace"))     kBehavior = Constants.KFOLD_NON_RANDOM;
			}
			
			if (optionsSet.contains("-deleterowstrain")) {
				index = options.indexOf("-deleterowstrain") + 1;
				while (!((String) options.get(index)).startsWith("-")) {
					String curr = (String) options.get(index);
					try {
						rowsToDeleteTrain.add(new Integer(Integer.parseInt(curr)));
						index++;
					} catch (NumberFormatException e) {
						//throw new ArgumentFormatException("Bad row to delete");
					}
				}
			}
			
			if (optionsSet.contains("-deletecolumnstrain")) {
				index = options.indexOf("-deletecolumnstrain") + 1;
				while (!((String) options.get(index)).startsWith("-")) {
					String curr = (String) options.get(index);
					try {
						colsToDeleteTrain.add(new Integer(Integer.parseInt(curr)));
						index++;
					} catch (NumberFormatException e) {
						//throw new ArgumentFormatException("Bad column to delete");
					}
				}
			}
			
			if (optionsSet.contains("-deleterowstest")) {
				index = options.indexOf("-deleterowstest") + 1;
				while (!((String) options.get(index)).startsWith("-")) {
					String curr = (String) options.get(index);
					try {
						rowsToDeleteTest.add(new Integer(Integer.parseInt(curr)));
						index++;
					} catch (NumberFormatException e) {
						//throw new ArgumentFormatException("Bad row to delete");
					}
				}
			}
			
			if (optionsSet.contains("-deletecolumnstest")) {
				index = options.indexOf("-deletecolumnstest") + 1;
				while (!((String) options.get(index)).startsWith("-")) {
					String curr = (String) options.get(index);
					try {
						colsToDeleteTest.add(new Integer(Integer.parseInt(curr)));
						index++;
					} catch (NumberFormatException e) {
						//throw new ArgumentFormatException("Bad column to delete");
					}
				}
			}
			
			if (optionsSet.contains("-deleterowsunlabeled")) {
				index = options.indexOf("-deleterowsunlabeled") + 1;
				while (!((String) options.get(index)).startsWith("-")) {
					String curr = (String) options.get(index);
					try {
						rowsToDeleteUnlabeled.add(new Integer(Integer.parseInt(curr)));
						index++;
					} catch (NumberFormatException e) {
						//throw new ArgumentFormatException("Bad row to delete");
					}
				}
			}
			
			if (optionsSet.contains("-deletecolumnsunlabeled")) {
				index = options.indexOf("-deletecolumnsunlabeled") + 1;
				while (!((String) options.get(index)).startsWith("-")) {
					String curr = (String) options.get(index);
					try {
						colsToDeleteUnlabeled.add(new Integer(Integer.parseInt(curr)));
						index++;
					} catch (NumberFormatException e) {
						//throw new ArgumentFormatException("Bad column to delete");
					}
				}
			}
			
			if (optionsSet.contains("-choosecolumn")) {
				index = options.indexOf("-normalize") + 1;
				s = (String) options.get(index);
				if (s.equals("true"))  normalizes[0] = true;
				if (s.equals("false")) normalizes[1] = true;
			}
			
			if (optionsSet.contains("-kernel")) {
				index = options.indexOf("-kernel") + 1;
				s = (String) options.get(index);
				if (s.equals("direct"))  kernels[0] = true;
				if (s.equals("linear")) kernels[1] = true;
				if (s.equals("polynomial"))  {
					kernels[2] = true;
					index = options.indexOf("-kernelparam") + 1;
					while (!((String) options.get(index)).startsWith("-")) {
						String curr = (String) options.get(index);
						try {
							polyParams.add(new Double(Double.parseDouble(curr)));
							index++;
						} catch (NumberFormatException e) {
							//throw new ArgumentFormatException("Bad column to delete");
						}
					}
				}
				if (s.equals("rbf"))  {
					kernels[3] = true;
					index = options.indexOf("-kernelparam") + 1;
					while (!((String) options.get(index)).startsWith("-")) {
						String curr = (String) options.get(index);
						try {
							rbfParams.add(new Double(Double.parseDouble(curr)));
							index++;
						} catch (NumberFormatException e) {
							//throw new ArgumentFormatException("Bad column to delete");
						}
					}
				}
				if (s.equals("cosine"))  kernels[4] = true;
			}
			
			if (optionsSet.contains("-bias")) {
				index = options.indexOf("-bias") + 1;
				s = (String) options.get(index);
				if (s.equals("false")) biases.add(new Double(0));
				else try {
					biases.add(new Double(s));
				} catch (NumberFormatException e) {
					//
				}
			}
			
			if (optionsSet.contains("-prior")) {
				index = options.indexOf("-prior") + 1;
				s = (String) options.get(index);
				if (s.equals("laplacian")) priors[0] = true;
				if (s.equals("gaussian"))  priors[1] = true;
				if (s.equals("none"))      priors[2] = true;
			}
			
			if (optionsSet.contains("-lambda")) {
				index = options.indexOf("-lambda") + 1;
				while (!((String) options.get(index)).startsWith("-")) {
					String curr = (String) options.get(index);
					try {
						lambdas.add(new Double(Double.parseDouble(curr)));
						index++;
					} catch (NumberFormatException e) {
						//throw new ArgumentFormatException("Bad column to delete");
					}
				}
			}
			
			if (optionsSet.contains("-algorithm")) {
				index = options.indexOf("-algorithm") + 1;
				s = (String) options.get(index);
				if (s.equals("component"))  updateRules[0] = true;
				if (s.equals("noncomponent")) updateRules[1] = true;
			}
			
			s = (String) options.get(options.indexOf("-displayweights") + 1);
			if (s.equals("true"))  learnDisplays[0] = true;
			else if (s.equals("false")) learnDisplays[0] = false;
			else learnDisplays[0] = Defaults.PREFS_LEARN_WEIGHTS;
			
			s = (String) options.get(options.indexOf("-displayiterations") + 1);
			if (s.equals("true"))  learnDisplays[1] = true;
			else if (s.equals("false")) learnDisplays[1] = false;
			else learnDisplays[1] = Defaults.PREFS_LEARN_ITERS;
			
			s = (String) options.get(options.indexOf("-displaybasisused") + 1);
			if (s.equals("true"))  learnDisplays[2] = true;
			else if (s.equals("false")) learnDisplays[2] = false;
			else learnDisplays[2] = Defaults.PREFS_LEARN_BASES;
			
			s = (String) options.get(options.indexOf("-displaytime") + 1);
			if (s.equals("true"))  learnDisplays[3] = true;
			else if (s.equals("false")) learnDisplays[3] = false;
			else learnDisplays[3] = Defaults.PREFS_LEARN_TIME;
			
			s = (String) options.get(options.indexOf("-displayprobabilities") + 1);
			if (s.equals("true"))  testDisplays[0] = true;
			else if (s.equals("false")) testDisplays[0] = false;
			else testDisplays[0] = Defaults.PREFS_TEST_PROBS;
			
			s = (String) options.get(options.indexOf("-displayclassifications") + 1);
			if (s.equals("true"))  testDisplays[1] = true;
			else if (s.equals("false")) testDisplays[1] = false;
			else testDisplays[1] = Defaults.PREFS_TEST_CLASSES;
			
			s = (String) options.get(options.indexOf("-displaynumwrong") + 1);
			if (s.equals("true"))  testDisplays[2] = true;
			else if (s.equals("false")) testDisplays[2] = false;
			else testDisplays[2] = Defaults.PREFS_TEST_WRONG;
			
			s = (String) options.get(options.indexOf("-displaynumcorrect") + 1);
			if (s.equals("true"))  testDisplays[3] = true;
			else if (s.equals("false")) testDisplays[3] = false;
			else testDisplays[3] = Defaults.PREFS_TEST_CORRECT;
			
			s = (String) options.get(options.indexOf("-displaypercentage") + 1);
			if (s.equals("true"))  testDisplays[4] = true;
			else if (s.equals("false")) testDisplays[4] = false;
			else testDisplays[4] = Defaults.PREFS_TEST_PERCENT;
			
			s = (String) options.get(options.indexOf("-displaymisclassifications") + 1);
			if (s.equals("true"))  testDisplays[5] = true;
			else if (s.equals("false")) testDisplays[5] = false;
			else testDisplays[5] = Defaults.PREFS_TEST_MISCLASS;
			
			
			s = (String) options.get(options.indexOf("-displaytotalcorrect") + 1);
			if (s.equals("true"))  crossDisplays[0] = true;
			else if (s.equals("false")) crossDisplays[0] = false;
			else crossDisplays[0] = Defaults.PREFS_CROSS_CORRECT;
			
			s = (String) options.get(options.indexOf("-displaytotalwrong") + 1);
			if (s.equals("true"))  crossDisplays[1] = true;
			else if (s.equals("false")) crossDisplays[1] = false;
			else crossDisplays[1] = Defaults.PREFS_CROSS_INCORRECT;
			
			s = (String) options.get(options.indexOf("-displaytotalpercentage") + 1);
			if (s.equals("true"))  crossDisplays[2] = true;
			else if (s.equals("false")) crossDisplays[2] = false;
			else crossDisplays[2] = Defaults.PREFS_CROSS_PERCENT;
			
			s = (String) options.get(options.indexOf("-displayaverageweights") + 1);
			if (s.equals("true"))  crossDisplays[3] = true;
			else if (s.equals("false")) crossDisplays[3] = false;
			else crossDisplays[3] = Defaults.PREFS_CROSS_AVGWEIGHT;
			
			s = (String) options.get(options.indexOf("-displayweightcounts") + 1);
			if (s.equals("true"))  crossDisplays[4] = true;
			else if (s.equals("false")) crossDisplays[4] = false;
			else crossDisplays[4] = Defaults.PREFS_CROSS_WEIGHTCOUNT;
			
			s = (String) options.get(options.indexOf("-displayaverageweightcounts") + 1);
			if (s.equals("true"))  crossDisplays[5] = true;
			else if (s.equals("false")) crossDisplays[5] = false;
			else crossDisplays[5] = Defaults.PREFS_CROSS_NONZERO;
			
			s = (String) options.get(options.indexOf("-displayconfusionmatrix") + 1);
			if (s.equals("true"))  crossDisplays[6] = true;
			else if (s.equals("false")) crossDisplays[6] = false;
			else crossDisplays[5] = Defaults.PREFS_CROSS_CONFUSION;
			
			s = (String) options.get(options.indexOf("-displayroccurve") + 1);
			if (s.equals("true"))  crossDisplays[7] = true;
			else if (s.equals("false")) crossDisplays[7] = false;
			else crossDisplays[5] = Defaults.PREFS_CROSS_ROC;
			
			
			if (optionsSet.contains("-saveto")) {
				index = options.indexOf("-saveto") + 1;
				saveFile = Core.parseFileName(options, index);
			}
			else saveFile = Constants.NO_FILE_LOADED;
			
			if (optionsSet.contains("-killthreshold")) {
				s = (String) options.get(options.indexOf("-killthreshold") + 1);
				try {
					killThresh = Double.parseDouble(s);
				} catch (NumberFormatException e) {
					killThresh = Defaults.PREFS_KILLTHRESH;
				}
			}
			if (optionsSet.contains("-convergencetol")) {
				s = (String) options.get(options.indexOf("-convergencetol") + 1);
				try {
					convTol = Double.parseDouble(s);
				} catch (NumberFormatException e) {
					convTol = Defaults.PREFS_CONVTOL;
				}
			}
			if (optionsSet.contains("-maxiterations")) {
				s = (String) options.get(options.indexOf("-maxiterations") + 1);
				try {
					maxIter = Integer.parseInt(s);
				} catch (NumberFormatException e) {
					maxIter = Defaults.PREFS_MAXITER;
				}
			}
			if (optionsSet.contains("-overrelaxed")) {
				s = (String) options.get(options.indexOf("-overrelaxed") + 1);
				if (s.equals("true")) overrelaxed = true;
				else if (s.equals("false")) overrelaxed = false;
				else overrelaxed = Defaults.PREFS_OVERRELAX;
			}
			if (optionsSet.contains("-cacheexp")) {
				s = (String) options.get(options.indexOf("-cacheexp") + 1);
				if (s.equals("true")) cacheExp = true;
				else if (s.equals("false")) cacheExp = false;
				else cacheExp = Defaults.PREFS_CACHEEXP;
			}
			if (optionsSet.contains("-cachekernel")) {
				s = (String) options.get(options.indexOf("-cachekernel") + 1);
				if (s.equals("false")) cacheKer = false;
				else if (s.equals("true")) cacheKer = true;
				else cacheKer = Defaults.PREFS_CACHEKERNEL;
			}
		}
		
		modPropTrain = new ModifyProperty();
		for (Iterator it = rowsToDeleteTrain.iterator(); it.hasNext();) {
			modPropTrain.getDeletedRows().add(new Integer(((Integer) it.next()).intValue() - 1));
		}
		for (Iterator it = colsToDeleteTrain.iterator(); it.hasNext();) {
			modPropTrain.getDeletedColumns().add(new Integer(((Integer) it.next()).intValue() - 1));
		}
		
		modPropTest = new ModifyProperty();
		for (Iterator it = rowsToDeleteTest.iterator(); it.hasNext();) {
			modPropTest.getDeletedRows().add(new Integer(((Integer) it.next()).intValue() - 1));
		}
		for (Iterator it = colsToDeleteTest.iterator(); it.hasNext();) {
			modPropTest.getDeletedColumns().add(new Integer(((Integer) it.next()).intValue() - 1));
		}
		
		modPropUnlabeled = new ModifyProperty();
		for (Iterator it = rowsToDeleteUnlabeled.iterator(); it.hasNext();) {
			modPropUnlabeled.getDeletedRows().add(new Integer(((Integer) it.next()).intValue() - 1));
		}
		for (Iterator it = colsToDeleteUnlabeled.iterator(); it.hasNext();) {
			modPropUnlabeled.getDeletedColumns().add(new Integer(((Integer) it.next()).intValue() - 1));
		}
		taskPanel.setOptions(task, trainingFile, classChoice, testFile, unlabeledFile, K, kBehavior);
		dataTrainPanel.setOptions(rowsToDeleteTrain, colsToDeleteTrain);
		dataUnlabeledPanel.setOptions(rowsToDeleteUnlabeled, colsToDeleteUnlabeled);
		dataTestPanel.setOptions(rowsToDeleteTest, colsToDeleteTest);
		methodsPanel.setOptions(normalizes, kernels, polyParams, rbfParams, biases, priors, lambdas, updateRules);
		choosePanel.setOptions(saveFile);
		preferencesFrame.getOutputFrame().setOptions(learnDisplays, testDisplays, crossDisplays);
		preferencesFrame.getTechnicalFrame().setOptions(killThresh, convTol, maxIter, overrelaxed, cacheExp, cacheKer);
	}
	
	public static void main(String args[]) {
		Graphical mainFrame = new Graphical();
		mainFrame.setVisible(true);
	}
}