package edu.duke.cs.smlr;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JWindow;
import javax.swing.UIManager;

/**A class that switches between loading the text and graphical interfaces.  Used so
 * that the JAR file has a single entrypoint.
 * @author Matt Edwards
 */

public class Main {
	
	/**Whether to attempt to load the native look-and-feel.  We'll leave it off
	 * until we've tested how this appears on Windows / Mac / etc.
	 * 
	 */
	private static boolean USE_NATIVE_LF = false;
	
	/**Number of milliseconds the splash screen will stay visible
	 */
	private static int SPLASH_VIEW_TIME = 1000;
	
	/**Number of milliseconds that will elapse before beginning the
	 * actual loading of the graphical interface
	 */
	private static final int WAIT_BEFORE_START = 600;
	
	/**Used as the entrypoint for the JAR file.  If passed any command line arguments, it will
	 * run the text interface.  If not, it will run the graphical interface.  <br>
	 * This allows <pre> java -jar smlr.jar </pre> to be the only needed command.
	 */
	public static void main(final String[] args) {		
		String temp = "/splash.jpg";
		boolean doText = true;
		for (int i = 0; i < args.length; ++i)		
			if (args[i].equals("native")) {
				USE_NATIVE_LF = true;
				args[i] = "";
			} else if (args[i].equals("gui")) {
				doText = false;
				args[i] = ""; 
			} else if (args[i].equals("longsplash")) {
				SPLASH_VIEW_TIME *= 10;
				args[i] = "";
			}
		final String splashfile = temp;
		
		if(doText && (args.length > 0))
			Text.main(args);
		else {
			if (USE_NATIVE_LF) {
				try {
					UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
				} catch (Exception e) {
					try {
						UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
					} catch (Exception e1) {}
				}
			}
			
			new Thread() {
				//thread to load the splash screen, allowing the app to load in the background
				public void run() {
					JWindow splash = new JWindow();                    
					
					//use the classloader to make it load from within a jar file
					ImageIcon picture = new ImageIcon(this.getClass().getResource(splashfile));                            
					
					JLabel label = new JLabel(picture);
					
					int w = picture.getIconWidth();
					int h = picture.getIconHeight();
					splash.setSize(w, h);
					splash.getContentPane().add(label, BorderLayout.CENTER);
					Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
					splash.setLocation(
							(dim.width - w) / 2,
							(dim.height - h) / 2);
					splash.setVisible(true);
					
					try {
						Thread.sleep(SPLASH_VIEW_TIME);
					} catch (InterruptedException e) {}					
					splash.dispose();				
				}
			}.start();	
			
			new Thread() {
				public void run() {
					try {
						Thread.sleep(WAIT_BEFORE_START);
					} catch (InterruptedException e) {}
					Graphical.main(args);
				}
			}.start();
		}
	}
	
	
}
