package edu.duke.cs.smlr.factories;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.PrintWriter;
import java.util.HashMap;
import java.util.Iterator;

import edu.duke.cs.smlr.util.HaltingException;

/**A class for centralizing file output and error-checking.  Using this
 * allows for writing to the same file within one program execution cycle
 * to append, not overwrite, old results (which is the behavior we want).
 * 
 * @author Matt Edwards, Jason Bosko
 */

public class FileStuff {

    /**A map of filenames (Strings) to PrintWriters.  <br>
     * Used to store already-opened streams so that, in the same
     * execution cycle, multiple learn tasks outputting to the same
     * file will append rather than overwrite.
     */
    private static HashMap outputFiles = new HashMap();
    
    private static final PrintWriter stderr = new PrintWriter(System.err); //i guess we could use the system.switchstandardout or whatever, but hey
    private static final PrintWriter stdout = new PrintWriter(System.out); //ditto
    
    /**
     * Gets the output stream associated with the file <b>filename</b>. 
     * If this is the first time <b>filename</b> has been requested, then a new output stream is created.
     * @param filename The name of the file to be opened.
     * @return A PrintWriter with the output stream associated with <b>filename</b>.
     * @throws HaltingException when the file cannot be opened
     */
    public static PrintWriter output(String filename) throws HaltingException {
       if(outputFiles.containsKey(filename)) 
            return (PrintWriter) outputFiles.get(filename);
        else {
            try {
                PrintWriter temp = new PrintWriter(new FileOutputStream(filename));
                //temp.println((new HTMLMaker()).getOpening());
                outputFiles.put(filename, temp);
                return temp;
            } catch (FileNotFoundException e) {
                throw new HaltingException("Can't create output file");
            }
        }        

//        return stdout;
    }
    
//    /**
//     * Adds the closing tags for an HTML file to all files that have previously been opened
//     *
//     */
//    public static void addClosings() {
//    	for (Iterator it = outputFiles.values().iterator(); it.hasNext();) {
//    		PrintWriter pw = (PrintWriter) it.next();
//    		pw.println((new HTMLMaker()).getClosing());
//    		pw.flush();
//    		pw.close();
//    	}
//    }
    
    public static PrintWriter error() {
        return stderr;
    }
    
    /*i think/hope finalize() for any streams does this all for us (minus logging)
    /**Flushes and closes all output streams that were accumulated by the program
     * <br>
     * Additionally, this checks to see whether any output errors occurred during the course
     * of the program.  If any did, it logs them in the error / log file.
     *
    public static void closeAllOff() {
        for(Iterator it = outputFiles.keySet().iterator(); it.hasNext(); ) {
            String s = (String) it.next();
            PrintWriter pw = (PrintWriter) outputFiles.get(s);
            if (pw.checkError()) error().println("Error encountered while writing to " + s);
            pw.close();
        }
        if(error().checkError()) error().println("Error encountered while writing to the log file");
        error().close();
        //stderr.close();
//        stdout.close();            
    }*/
}



