package edu.duke.cs.smlr.factories;

import edu.duke.cs.smlr.learn.interfaces.IKernel;
import edu.duke.cs.smlr.learn.kernels.CosineKernel;
import edu.duke.cs.smlr.learn.kernels.DirectKernel;
import edu.duke.cs.smlr.learn.kernels.LinearKernel;
import edu.duke.cs.smlr.learn.kernels.PolynomialKernel;
import edu.duke.cs.smlr.learn.kernels.RBFKernel;
import edu.duke.cs.smlr.util.Constants;


/**A factory for creating kernel classes.
 * 
 * @author Matt Edwards, Jason Bosko
 */

public class KernelFactory {

    /**
     * Returns the kernel specified by <b>whichKernel</b>. 
     * The integer code for each kernel is located in edu.duke.cs.smlr.util.Constants
     * @param whichKernel Specifies which kernel to use. 
     * @return The kernel specified by <b>whichKernel</b>
     */
	public static IKernel makeKernel(int whichKernel) {
        switch (whichKernel) {
		case Constants.KERNEL_COSINE:
			return new CosineKernel();			
		case Constants.KERNEL_DIRECT:
			return  new DirectKernel();			
		case Constants.KERNEL_LINEAR:
			return new LinearKernel();			
		case Constants.KERNEL_POLYNOMIAL:
			return new PolynomialKernel();			
		case Constants.KERNEL_RBF:
			return new RBFKernel();			
		default:
			throw new IllegalArgumentException("Unrecognized kernel");
		}        
    }

}
