package edu.duke.cs.smlr.factories;

import edu.duke.cs.smlr.learn.interfaces.ILearner;
import edu.duke.cs.smlr.learn.learners.ArrayStepwiseLearner;
import edu.duke.cs.smlr.learn.learners.IterativeLearner;
import edu.duke.cs.smlr.util.Constants;

/**A factory for creating learner classes.
 * 
 * @author Matt Edwards, Jason Bosko
 */

public class LearnerFactory {
    
	/**
     * Returns the learner specified by <b>whichLearner</b>. 
     * The integer code for each learner is located in edu.duke.cs.smlr.util.Constants
     * @param whichLearner Specifies which learner to use. 
     * @return The learner specified by <b>whichLearner</b>
     */
	public static ILearner makeLearner(int whichLearner) {
        
		switch (whichLearner) {
		case Constants.ALGORITHM_NON_COMPONENT:
			return new IterativeLearner();
		case Constants.ALGORITHM_COMPONENT:
			return new ArrayStepwiseLearner();
		default:
			throw new IllegalArgumentException("Unrecognized method");
		}        
    }    
}
