package edu.duke.cs.smlr.gui;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;

import edu.duke.cs.smlr.Graphical;
import edu.duke.cs.smlr.gui.help.Help;
import edu.duke.cs.smlr.util.Messages;

/**
 * Class to create the menus for the GUI.
 * 
 * @author Matt Edwards, Jason Bosko
 */
public class MenuMaker {
	/**
	 * Makes all the menus needed for the GUI.
	 * @param parent The GUI.
	 */
	public static void makeMenus(JFrame parent) {
	    JMenuBar bar = new JMenuBar();

	    bar.add(makeFileMenu(parent));
	    
	    // Check to see if this is running on a Mac (no preferences in the menu needed for a Mac)
	    if(!(((Graphical) parent).isMacOSX)) {
	        bar.add(makePreferencesMenu(parent));
	    }
	    bar.add(makeHelpMenu(parent));
	    parent.setJMenuBar(bar);
	}
	
	/**
	 * Makes the "File" menu.
	 * @param parent The GUI
	 * @return the "File" menu
	 */
	private static JMenu makeFileMenu(final JFrame parent) {
		JMenu fileMenu = new JMenu(Messages.MENU_FILE);
		fileMenu.setMnemonic('F');
		
		fileMenu.add(new JMenuItem() {
			public int getMnemonic() {
				return 'O';
			}
		}).setAction(new AbstractAction("Open Settings") {
			public void actionPerformed(ActionEvent evt) {
				JFileChooser fc = edu.duke.cs.smlr.util.Utilities.newFileChooser();	        	
				if (fc.showOpenDialog(parent) == JFileChooser.APPROVE_OPTION) {
					File file = fc.getSelectedFile();
					((Graphical) parent).loadSettings(file.getPath());
				}
				edu.duke.cs.smlr.util.Utilities.setDirectory(fc.getCurrentDirectory());
			}
		});
		
		fileMenu.add(new JMenuItem() {
			public int getMnemonic() {
				return 'S';
			}
		}).setAction(new AbstractAction("Save Settings") {
			public void actionPerformed(ActionEvent evt) {
				JFileChooser fc = edu.duke.cs.smlr.util.Utilities.newFileChooser();
				if (fc.showSaveDialog(parent) == JFileChooser.APPROVE_OPTION) {
					File file = fc.getSelectedFile();
					((Graphical) parent).saveSettings(file.getPath(), true);
				}
				edu.duke.cs.smlr.util.Utilities.setDirectory(fc.getCurrentDirectory());
			}
		});
				
	    if(!(((Graphical) parent).isMacOSX)) {
	        fileMenu.addSeparator();
	        fileMenu.add(new JMenuItem() {
	            public int getMnemonic() {
	                return 'X';
	            }
	        }).setAction(new AbstractAction("Exit") {
	            public void actionPerformed(ActionEvent e) {
	                System.exit(0);
	            }			
	        });
	    }
	    return fileMenu;
	}
	
	/**
	 * Makes the "Preferences" menu.
	 * @param parent The GUI
	 * @return the "Preferences" menu
	 */
	private static JMenu makePreferencesMenu(final JFrame parent) {
	    JMenu preferencesMenu = new JMenu("Options");
	    preferencesMenu.setMnemonic('P');
	    preferencesMenu.add(new JMenuItem() {
	        public int getMnemonic() {
	            return 'P';
	        }			
	    }).setAction(new AbstractAction("Preferences") {
	        public void actionPerformed(ActionEvent evt) {
	            ((Graphical) parent).showPreferencesFrame();
	        }
	    });		
		
		return preferencesMenu;
	}
	
	/**
	 * Makes the "Help" menu.
	 * @param parent The GUI
	 * @return the "Help" menu
	 */
	private static JMenu makeHelpMenu(final JFrame parent) {
		JMenu helpMenu = new JMenu("Help");
		helpMenu.setMnemonic('H');
		
		helpMenu.add(new JMenuItem() {
			public int getMnemonic() {
				return 'H';
			}			
		}).setAction(new AbstractAction("Help Topics") {
			public void actionPerformed(ActionEvent evt) {
				Help help = new Help();
				help.createHelpDisplay();
			}
		});
		
		helpMenu.add(new JMenuItem() {
			public int getMnemonic() {
				return 'L';
			}		   
		}).setAction(new AbstractAction("License Information") {
			public void actionPerformed(ActionEvent evt) {
				JFrame licensing = new JFrame("License Information");
				licensing.setSize(550, 500);
				try {					
					JEditorPane licenseArea = new JEditorPane(this.getClass().getResource("/license.html"));
					licenseArea.setContentType("text/html");
					//licenseArea.scrollRectToVisible(licenseArea.getBounds());
							
					licenseArea.setEditable(false);
					licensing.getContentPane().add(new JScrollPane(licenseArea));
				} catch (IOException e) {
					//System.err.println("Error loading license");
					licensing.getContentPane().add(new JLabel("Please visit www.cs.duke.edu for licensing information"));
					//do something if we can't load licensing
				}
				Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
				licensing.validate();
				licensing.setLocation(
				          (dim.width - licensing.getWidth()) / 2, 
				          (dim.height - licensing.getHeight()) / 2);
				licensing.setVisible(true);
			}
		});
		
		helpMenu.add(new JMenuItem() {
			public int getMnemonic() {
				return 'A';
			}			
		}).setAction(new AbstractAction("About") {
			public void actionPerformed(ActionEvent evt) {
				((Graphical) parent).showAboutFrame();
			}			
		});
		
		return helpMenu;
	}
	
}
