package edu.duke.cs.smlr.gui.frames;

import java.awt.Dimension;
import java.awt.Toolkit;

import javax.swing.JFrame;

import edu.duke.cs.smlr.Graphical;

/**The base class for the frame classes used in the graphical interface.
 * All frames used in the graphical interface inherit from this class, which
 * provides a few convenience methods used by many frames.  This class contains
 * a JFrame as a protected member, but does not inherit from it directly. 
 * 
 * @see JFrame
 *  
 * @author Matt Edwards, Jason Bosko
 *
 */
public class AbstractFrame {

	protected JFrame frame;
	protected Graphical parent;
	
	/**This is hack to let certain frames use the content of other frames without
	 * the other frames being instantiated.  See how the PreferencesFrame is composed
	 * of the contents of OutputPreferencesFrame and TechnicalPreferencesFrame, for
	 * example. 
	 */
	protected AbstractFrame() {
		
	}
	
	public AbstractFrame(Graphical p) {
		frame = new JFrame();
		parent = p;
	}
	
	/**
	 * Centers the frame on the screen.
	 *
	 */
	public void center() {
	       Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
			frame.setLocation(
			        (dim.width - frame.getWidth()) / 2,
			        (dim.height - frame.getHeight()) / 2);
	}
	
	/**
	 * Sets the visibility of the frame to <b>b</b>.
	 * @param b Whether the frame should be visible or not.
	 */
	public void setVisible(boolean b) {
		frame.setVisible(b);
	}
}
