package edu.duke.cs.smlr.gui.frames;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Iterator;
import java.util.TreeMap;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;

import edu.duke.cs.smlr.gui.table.ColoredRenderer;
import edu.duke.cs.smlr.util.ClassificationMap;

/**
 * Class to show the key for the data panel.
 * @author Jason Bosko, Matt Edwards
 */

public class KeyFrame extends AbstractFrame {
	
	private JLabel temp;
	
    /**
     * Constructs a KeyFrame from the information in a ClassificationMap, <b>map</b>
     * @param map The ClassificationMap with the information about the class labels.
     */
	public KeyFrame(ClassificationMap map) {
    	super(null); //we don't care about using the parent
    	frame.setTitle("Key");
        frame.setSize(300, 300);
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);        
        frame.getContentPane().setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();

        /* do this so we display keys in some sort of attractive order.
         * this shouldn't fail since adaptive doesn't read much besides 
         * double/strings/integers/whatever (comparable stuff)
         */        
        TreeMap sortedMap = new TreeMap(map.getOldToNew());
        
        this.center();
        
        c.insets = new Insets(10, 30, 10, 30);
        c.gridx = 0;
        c.gridy = 0;
        //c.anchor = GridBagConstraints.WEST;
        frame.getContentPane().add(new JLabel("Color"), c);
        
        c.gridx = 1;
        c.gridy = 0;
        //c.anchor = GridBagConstraints.EAST;
        frame.getContentPane().add(new JLabel("Class Label"), c);
        
        if (map.getTotalClasses() == 2) {
        	c.gridx = 2;
        	frame.getContentPane().add(new JLabel("Interpretation for ROC curve"), c);
        }
        
        int i = 1;
        for(Iterator it = sortedMap.keySet().iterator(); it.hasNext();) {
            Double index = (Double) map.getOldToNew().get(it.next());
            
            Color col = ColoredRenderer.colors[(index.intValue() - 1)%ColoredRenderer.maxColors];
            
            temp = new JLabel();
            temp.setOpaque(true);
            temp.setBackground(col);
            temp.setBorder(BorderFactory.createLineBorder(Color.BLACK));
            temp.setText("          ");    
            c.gridx = 0;
            c.gridy = i;
            //c.anchor = GridBagConstraints.WEST;
            frame.getContentPane().add(temp, c);
            //tempPanel.add(temp, BorderLayout.WEST);
            
            temp = new JLabel();
            temp.setText(map.getOriginalClassification(index).toString());
            c.gridx = 1;
            c.gridy = i;
            //c.anchor = GridBagConstraints.EAST;
            frame.getContentPane().add(temp, c);
            
            if (map.getTotalClasses() == 2) {
            	temp = new JLabel();
            	if (i == 1) temp.setText("Negative");
            	else temp.setText("Positive");
            	c.gridx = 2;
            	c.gridy = i;
            	frame.getContentPane().add(temp, c);
            }
            i++;
            //tempPanel.add(temp, BorderLayout.EAST);
            
            //this.getContentPane().add(tempPanel);
        }
        
        frame.getContentPane().validate();
        frame.validate();
        frame.setVisible(true);
        frame.pack();
    }
}
