package edu.duke.cs.smlr.gui.frames;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.BoxLayout;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;

import edu.duke.cs.smlr.Graphical;

/**
 * A frame that contains both the output preferences frame and the technical preferences frame, and puts them in the 
 * same frame, but controlled by tabs.
 * 
 * @author Jason Bosko, Matt Edwards
 *
 */
public class PreferencesFrame extends AbstractFrame {
	OutputPreferencesFrame o;
	TechnicalPreferencesFrame t;
	
	/**
	 * Constructs a Preferences frame for the GUI
	 * @param p The GUI
	 */
	public PreferencesFrame(Graphical p) {
		super(p);

		JTabbedPane jt = new JTabbedPane();

        frame.setTitle("Preferences");
        frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
        frame.setSize(630, 450);
        frame.getContentPane().add(jt);		

        o = new OutputPreferencesFrame(0, parent);
        t = new TechnicalPreferencesFrame(0, parent);
		
        JPanel oPanel = new JPanel();
        JPanel tPanel = new JPanel();
		
        tPanel.setLayout(new BoxLayout(tPanel, BoxLayout.Y_AXIS));
        tPanel.add(t.createConvergencePanel());
        tPanel.add(t.createPerformancePanel());
        
        oPanel.setLayout(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.BOTH;
		
        c.gridx = 0;
        c.weightx = 1;
        c.gridy = 0;
        c.weighty = 1;
        oPanel.add(o.createLearnResultDisplayPanel(), c);

        c.gridy = 1;
        oPanel.add(o.createTestResultDisplayPanel(), c);
		
        c.gridy = 2;
        oPanel.add(o.createKFoldResultsPanel(), c);        

        jt.addTab("Output", null, oPanel, "Preferences for formatting the output");
        jt.addTab("Technical", null, tPanel, "Preferences for technical aspects of the algorithm");
        frame.getContentPane().add(jt);
		
        center();
	}

	/**
	 * Gets the output frame within this frame.
	 * @return The output frame
	 */
	public OutputPreferencesFrame getOutputFrame() {
		return o;
	}

	/**
	 * Gets the techinacal frame within this frame.
	 * @return The technical frame
	 */
	public TechnicalPreferencesFrame getTechnicalFrame() {
		return t;
	}

}
