package edu.duke.cs.smlr.gui.help;


import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;

import edu.duke.cs.smlr.util.Messages;

/**
 * A static class that controls the frames used to display help messages
 * 
 * @author Jason Bosko, Matt Edwards
 */
public class Help {
	
	private static JEditorPane helpArea;

	public static final int TASK = 0;
	public static final int TRAINING_DATA = 1;
	public static final int EVALUATION_OPTIONS = 2;
	public static final int USE_OPTIONS = 3;
	public static final int OBSERVATIONS = 4;
	public static final int DELETE_ROW_COLUMN = 5;
	public static final int DATA_WINDOW = 6;
	public static final int NORMALIZE = 7;
	public static final int KERNELS = 8;
	public static final int BIAS = 9;
	public static final int PRIORS = 10;
	public static final int SPARSITY = 11;
	public static final int UPDATE_RULE = 12;
	public static final int LEARN_DISPLAY = 13;
	public static final int TEST_DISPLAY = 14;
	public static final int KFOLD_DISPLAY = 15;
	public static final int CONVERGENCE_SETTINGS = 16;
	public static final int PERFORMANCE_SETTINGS = 17;
	public static final int CHOOSE_PANEL = 18;
	public static final int SAVE_AND_LEARN = 19;
	
	private static final int helpTotal = 20;
	
	private static final int LEFT_WIDTH = 100;
	private static final int RIGHT_WIDTH = 600;
	private static final int HEIGHT = 450;
	
	static String getHelp(int h) {
	    switch(h) {
	    case TASK                 : return Messages.TASK_HELP;
	    case TRAINING_DATA        : return Messages.TRAINING_DATA_HELP;
	    case EVALUATION_OPTIONS   : return Messages.EVALUATION_OPTIONS_HELP;
	    case USE_OPTIONS          : return Messages.USE_OPTIONS_HELP;
	    case OBSERVATIONS         : return Messages.OBSERVATIONS_HELP;
	    case DELETE_ROW_COLUMN    : return Messages.DELETE_ROW_COLUMN_HELP;
	    case DATA_WINDOW          : return Messages.DATA_WINDOW_HELP;
	    case NORMALIZE            : return Messages.NORMALIZE_HELP;
	    case KERNELS              : return Messages.KERNELS_HELP;
	    case BIAS                 : return Messages.BIAS_HELP;
	    case PRIORS               : return Messages.PRIORS_HELP;
	    case SPARSITY             : return Messages.SPARSITY_HELP;
	    case UPDATE_RULE          : return Messages.UPDATE_RULE_HELP;
	    case LEARN_DISPLAY        : return Messages.LEARN_DISPLAY_HELP;
	    case TEST_DISPLAY         : return Messages.TEST_DISPLAY_HELP;
	    case KFOLD_DISPLAY        : return Messages.KFOLD_DISPLAY_HELP;
	    case CONVERGENCE_SETTINGS : return Messages.CONVERGENCE_SETTINGS_HELP;
	    case PERFORMANCE_SETTINGS : return Messages.PERFORMANCE_SETTINGS_HELP;
	    case CHOOSE_PANEL         : return Messages.CHOOSE_PANEL_HELP;
	    case SAVE_AND_LEARN       : return Messages.SAVE_AND_LEARN_HELP;
	    default                   : return "Unrecognized help constant";
	    }
	}
	
	static String toString(int h) {
	    switch (h) {
	    case TASK                 : return "Task";
	    case TRAINING_DATA        : return "Training Data";
	    case EVALUATION_OPTIONS   : return "Evaluation Options";
	    case USE_OPTIONS          : return "Testing Options";
	    case OBSERVATIONS         : return "Observations";
	    case DELETE_ROW_COLUMN    : return "Delete Row/Column";
	    case DATA_WINDOW          : return "Data Window";
	    case NORMALIZE            : return "Normalize";
	    case KERNELS              : return "Kernels";
	    case BIAS                 : return "Bias";
	    case PRIORS               : return "Priors";
	    case SPARSITY             : return "Sparsity";
	    case UPDATE_RULE          : return "Update Rule";
	    case LEARN_DISPLAY        : return "Learn Display";
	    case TEST_DISPLAY         : return "Test Display";
	    case KFOLD_DISPLAY        : return "K-Fold Display";
	    case CONVERGENCE_SETTINGS : return "Convergence Settings";
	    case PERFORMANCE_SETTINGS : return "Performance Settings";
	    case CHOOSE_PANEL         : return "Choose Panel";
	    case SAVE_AND_LEARN       : return "Save Settings and Learn";
	    default                   : return "Unrecognized help constant";
	    }
	}
	
	/**
	 * Creates the frame that displays the general help with all topics.
	 *
	 */
	public void createHelpDisplay() {
		JFrame helpDisplay = new JFrame();
		helpDisplay.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		helpDisplay.setTitle("Help Topics");
		helpDisplay.setSize(LEFT_WIDTH + RIGHT_WIDTH, HEIGHT);
		
		helpDisplay.getContentPane().setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		JScrollPane labelPane = new JScrollPane(getHelpLabels(), JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		JScrollPane helpPane = new JScrollPane(getHelpArea(-1), JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		
		labelPane.setPreferredSize(new Dimension(LEFT_WIDTH, HEIGHT));
		helpPane.setPreferredSize(new Dimension(RIGHT_WIDTH, HEIGHT));
		
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		helpDisplay.setLocation(
		        (dim.width - helpDisplay.getWidth()) / 2,
		        (dim.height - helpDisplay.getHeight()) / 2);
				
		c.fill = GridBagConstraints.BOTH;
		c.weighty = 1;
		c.gridx = 0;
		c.weightx = 1;
		helpDisplay.getContentPane().add(labelPane, c);
		
		c.gridx = 1;
		c.weightx = 2;
		helpDisplay.getContentPane().add(helpPane, c);
		
		helpDisplay.setVisible(true);
	}
	
	/**
	 * Creates a help display frame specific to the help topic specified by <b>helpConstant</b>.
	 * @param helpConstant The constant associated with the specific type of help requested
	 */
	public static void createHelpDisplay(int helpConstant) {
		JFrame helpDisplay = new JFrame();
		helpDisplay.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		helpDisplay.setTitle("Help Item");
		
		helpArea = getHelpArea(helpConstant);
		helpArea.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
		
		JScrollPane helpScrollPane = new JScrollPane(helpArea, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);		
		helpScrollPane.setPreferredSize(new Dimension(RIGHT_WIDTH, HEIGHT));
		helpScrollPane.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createEmptyBorder(5,5,5,5), BorderFactory.createLoweredBevelBorder()));
		helpDisplay.getContentPane().add(helpScrollPane);
		helpDisplay.pack();
		
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		helpDisplay.setLocation(
		        (dim.width - helpDisplay.getWidth()) / 2,
		        (dim.height - helpDisplay.getHeight()) / 2);
		
		helpDisplay.setVisible(true);
	}
	
	/**
	 * Creates a panel that contains a list of the labels for all the different help topics.
	 * @return The panel with all the labels for the different help topics
	 */
	private static JPanel getHelpLabels() {
		JPanel p = new JPanel();
		p.setLayout(new BoxLayout(p, BoxLayout.Y_AXIS));
		p.setBackground(Color.WHITE);
		p.setPreferredSize(new Dimension(LEFT_WIDTH, HEIGHT));
		JLabel helpLabels[] = new JLabel[helpTotal];
		for (int i = 0; i < helpTotal; ++i) {
			final int index = i;
			helpLabels[index] = new JLabel(Help.toString(i));
			helpLabels[index].addMouseListener(new MouseListener() {
				public void mouseClicked(MouseEvent arg0) {
					helpArea.setText("<b><u>" + 
					                 Help.toString(index) + 
					                 "</b></u><br><br>" + 
					                 getHelp(index));
				}
				//Don't care about these
				public void mousePressed(MouseEvent arg0) { }
				public void mouseReleased(MouseEvent arg0) { }
				public void mouseEntered(MouseEvent arg0) { }
				public void mouseExited(MouseEvent arg0) { }
				
			});
			
			p.add(helpLabels[index]);
		}
		return p;
	}
	
	/**
	 * Creates the text area specific to the help topic associated with <b>helpConstant</b>.
	 * @param helpConstant The constant associated with the type fo help requested.
	 * @return The text area specific to the help topic.
	 */
	private static JEditorPane getHelpArea(int helpConstant) {
	    if (helpConstant>=0) {
	        helpArea = new JEditorPane("text/html", "<b><u>" + 
	                                                Help.toString(helpConstant) + 
	                                                "</b></u><br><br>" + 
	                                                getHelp(helpConstant));
	    } else {
	        helpArea = new JEditorPane("text/html", "");
	    }
		helpArea.setPreferredSize(new Dimension(RIGHT_WIDTH, HEIGHT));
		helpArea.setEditable(false);
		return helpArea;
	}
}
