//TODO: if the user decides to delete all of one class, the displayed M won't update / more bad stuff will happen


package edu.duke.cs.smlr.gui.panels;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.MouseEvent;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.TreeSet;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import javax.swing.table.TableColumn;

import edu.duke.cs.smlr.Graphical;
import edu.duke.cs.smlr.gui.frames.KeyFrame;
import edu.duke.cs.smlr.gui.help.Help;
import edu.duke.cs.smlr.gui.help.HelpButton;
import edu.duke.cs.smlr.gui.table.AlternatingRowRenderer;
import edu.duke.cs.smlr.gui.table.ColoredRenderer;
import edu.duke.cs.smlr.gui.table.UneditableModel;
import edu.duke.cs.smlr.learn.Modifier;
import edu.duke.cs.smlr.learn.interfaces.IMatrix;
import edu.duke.cs.smlr.learn.properties.ModifyProperty;
import edu.duke.cs.smlr.util.AdaptiveReader;
import edu.duke.cs.smlr.util.ClassificationMap;
import edu.duke.cs.smlr.util.Constants;
import edu.duke.cs.smlr.util.Defaults;
import edu.duke.cs.smlr.util.Messages;

/**
 * Creates a panel for showing and editing the data.
 * @author Matt Edwards, Jason Bosko
 */
public class DataPanel extends AbstractPanel implements FocusListener, ActionListener {
    
	JTabbedPane tab = new JTabbedPane();
	
    private JPanel unloadedPanel;
    private JPanel loadingPanel;
    private JPanel loadedPanel;
    
    private JTable dataTable;
    private JScrollPane scrollPane;
    
    private IMatrix originalData; // we'll keep the original data so that "refresh" will start fresh each time
    private IMatrix currData;
    private IMatrix originalClasses; // same for classifications
    private IMatrix currClasses;
    private String oldState;
    private ModifyProperty modifyProp;
    
    private JLabel featureCount;
    private JLabel dataCount;
    private JLabel classesCount;
    
    private int m;
    
    ClassificationMap map;
    
    private ArrayList rowDel;
    private ArrayList colDel;
    
    private JTextField delRowField;
    private JTextField delColField;
    
    private JTextField viewOptions[];
    private JLabel viewOptionsLabels[];
    private final String viewOptionsTitles[] = { "Rows to View", "Start Row", "Columns to View", "Start Column"};
    private final int numViewOptions = 4;
    
    private int rowsToView;
    private int columnsToView;
    private int startRowVal;
    private int startColVal;
    
    private JButton viewKeyButton;
    private JButton viewAllButton;
    
    /**
     * Constructs the data panel for the GUI.
     * @param parent The GUI
     */
    public DataPanel(Graphical parent) {
        super(parent);
        panel.setLayout(new BorderLayout());
        //rowsToView = columnsToView = 10;
        //startRowVal = startColVal = 1;
        rowsToView = Defaults.VIEW_ROWS_TOTAL;
        columnsToView = Defaults.VIEW_COLS_TOTAL;
        startRowVal = Defaults.VIEW_ROWS_START;
        startColVal = Defaults.VIEW_COLS_START;
        oldState = "random string";
        modifyProp = new ModifyProperty();
        scrollPane = new JScrollPane();
        rowDel = new ArrayList();
        colDel = new ArrayList();
        
        unloadedPanel = new JPanel(new BorderLayout());
        JLabel label = new JLabel(Messages.DATA_NOT_LOADED);
        label.setHorizontalAlignment(JLabel.CENTER);
        unloadedPanel.add(label);
        
        loadingPanel = new JPanel(new BorderLayout());
        
        JLabel label2 = new JLabel(Messages.DATA_LOADING);
        label2.setHorizontalAlignment(JLabel.CENTER);
        loadingPanel.add(label2);
        
        JPanel northPanel = new JPanel();
        northPanel.setLayout(new BoxLayout(northPanel, BoxLayout.Y_AXIS));
        northPanel.add(createCountsPanel());
        northPanel.add(createButtons());
        northPanel.add(createViewOptions());
        
        JPanel southPanel = new JPanel();
        southPanel.setLayout(new BoxLayout(southPanel, BoxLayout.Y_AXIS));
        southPanel.add(createRowColDeletePanel());
        //southPanel.add(createViewOptions());
        
        loadedPanel = new JPanel(new BorderLayout());
        loadedPanel.add(northPanel, BorderLayout.NORTH);
        loadedPanel.add(scrollPane, BorderLayout.CENTER);
        loadedPanel.add(southPanel, BorderLayout.SOUTH);;
    }
    
    /**Shows the message that no data has been loaded.
     * 
     */
    public void showUnloadedPanel() {
        panel.removeAll();
        panel.add(unloadedPanel, BorderLayout.CENTER);
        panel.validate();	
        panel.repaint();
    }
    
    /**Shows an intermediate message indicating that data is in the process
     * of being loaded.
     * 
     */
    public void showLoadingPanel() {
        panel.removeAll();
        panel.add(loadingPanel, BorderLayout.CENTER);
        panel.validate();
        panel.repaint();
    }
    
    /**Shows the data as a table along with its statistics and buttons for
     * manipulating it.
     * 
     */
    public void showLoadedPanel() {
        panel.removeAll();
        panel.add(loadedPanel, BorderLayout.CENTER);
        panel.validate();
        panel.repaint();
    }
    
    /**
     * Shows the key for which colors are which classes.
     *
     */
    public void showKey() {
        new KeyFrame(map);
    }
    
    /**
     * loads the table into memory from a file
     * @param fileName the file path
     * @param classChoice the column containing the class labels
     * @param mp the ModifyProperty that explains how to manipulate the data
     */
    public void loadTable(String fileName, int classChoice, ModifyProperty mp) {
        AdaptiveReader myReader = new AdaptiveReader();
        
        try {
        	myReader.read(fileName, classChoice);
        	originalData = myReader.getX();
        	if (classChoice == Constants.READER_UNCLASSIFIED) {
        		m = 0;
        		originalClasses = null;
        		map = null;
        	}
        	else {
        		m = myReader.getM();
                originalClasses = myReader.getY();
                map = myReader.getClassificationMap();
        	}
            oldState = "another random string"; // force a reload
            loadTable(originalData, originalClasses, mp);
        } catch (FileNotFoundException e) {
            JOptionPane.showMessageDialog(panel,
                    Messages.ERROR_DATA_LOAD,
                    Messages.ERROR_FILE_FORMAT_TITLE, JOptionPane.ERROR_MESSAGE);
            showUnloadedPanel();
        }
    }
    
    /**
     * prepares the data members to be loaded into the table
     * @param d the data features to be loaded
     * @param c the classifications to be loaded
     * @param mp the ModifyProperty that explains how to manipulate the data
     */
    public void loadTable(IMatrix d, IMatrix c, ModifyProperty mp) {
        oldState = Integer.toHexString((int) Math.random() * 1000); // uh... I like hex
        currData = d;
        currClasses = c;
        if (featureCount != null)
            featureCount.setText("Features: " + currData.numCols());
        if (dataCount != null)
            dataCount.setText("Examples: " + currData.numRows());
        if (classesCount != null)
            classesCount.setText("Classes: " + (m <= 0 ? "unlabeled" : Integer.toString(m)));
        loadTable(mp);
    }
    
    /**
     * displays the data in the panel
     * @param mp the ModifyProperty that explains how to manipulate the data
     */
    public void loadTable(ModifyProperty mp) {
        /*
         * these next lines make the method only execute if relevant input has
         * changed... this simplifies other parts of the code (listener
         * handlers) and I think/hope the cost of this computation is
         * [significantly] smaller than redrawing unnecessarily on every focus
         * event
         */
        if (mp != null) {
            currData = Modifier.modifyData(originalData, mp);
            if (originalClasses != null) currClasses = Modifier.modifyClasses(originalClasses, mp);
            updateCounts();
            oldState = "Garbage";
        }
        String state = Integer.toHexString(startRowVal)
        + Integer.toHexString(startColVal)
        + Integer.toHexString(rowsToView)
        + Integer.toHexString(columnsToView);
        
        if (state.equals(oldState))
            return;
        else
            oldState = state;
        
        int rows = Math.min(rowsToView, currData.numRows() - startRowVal+1);
        int columns = Math.min(columnsToView+1, currData.numCols() + 2
                - startColVal);
        if (originalClasses == null) columns--;
        Object[][] tab = new Double[rows][columns];
        String[] names = new String[columns];
        if (columns > 0 && originalClasses != null)
            names[names.length - 1] = "Classification";
        if (originalClasses != null)
        	for (int i = 0; i < columns - 1; ++i)
        		names[i] = "Feature " + Integer.toString(i + startColVal);
        else
        	for (int i = 0; i < columns; ++i)
        		names[i] = "Feature " + Integer.toString(i + startColVal);
        
        if (columns > 0 && originalClasses != null)
            for (int i = 0; i < rows; ++i)
                tab[i][names.length - 1] = new Double(currClasses.get(i + (startRowVal-1), 0));
        
        if (originalClasses != null)
        	for (int i = 0; i < rows; ++i)
        		for (int j = 0; j < columns - 1; ++j)
        			tab[i][j] = new Double(currData.get(i + (startRowVal-1), j
        					+ (startColVal-1)));
        else
        	for (int i = 0; i < rows; ++i)
        		for (int j = 0; j < columns; ++j)
        			tab[i][j] = new Double(currData.get(i + (startRowVal-1), j
        					+ (startColVal-1)));
        
        dataTable = new JTable(new UneditableModel(tab, names)) {
            public String getToolTipText(MouseEvent e) {
                int rowIndex = rowAtPoint(e.getPoint());
                int colIndex = convertColumnIndexToModel(columnAtPoint(e.getPoint())) + 1;
                
                if (colIndex == this.getColumnCount()) return null; //no tooltip for classification column
                if (originalClasses != null) return "Data Point " + (rowIndex + 1) + ", Class " + map.getOriginalClassification((Double) getValueAt(rowIndex, convertColumnIndexToView(this.getColumnCount() - 1))) + ", Feature " + colIndex;
                else return "Data Point " + (rowIndex + 1) + ", Class unlabeled, Feature " + colIndex;
            }
            
        };
        
        if (originalClasses != null) dataTable.setDefaultRenderer(Double.class, new ColoredRenderer());
        else dataTable.setDefaultRenderer(Double.class, new AlternatingRowRenderer());
        
        for (int i = 0; i < columns; i++) {
            TableColumn co = dataTable.getColumnModel().getColumn(i);
            if(i == dataTable.getColumnCount() - 1 && originalClasses != null) {
                co.setMaxWidth(0);
                co.setPreferredWidth(0);
                co.setResizable(false);
                co.setWidth(0);
                co.setHeaderValue("");
            } else co.setPreferredWidth(100);
        }
        dataTable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        if (scrollPane != null)
            loadedPanel.remove(scrollPane);
        scrollPane = new JScrollPane(dataTable);
        scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        loadedPanel.add(scrollPane, BorderLayout.CENTER);
        loadedPanel.validate();
    }
    
    /**
     * Creates the panel with the options for deleting rows/columns of the data.
     * 
     * @return The panel for deleting rows/columns of the data.
     */
    public JPanel createRowColDeletePanel() {
        JPanel delRowColPanel = new JPanel(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        Border blackline = BorderFactory.createLineBorder(Color.black);
        TitledBorder title = BorderFactory.createTitledBorder(blackline,
        		"Remove Examples or Features from Consideration");
        title.setTitleJustification(TitledBorder.CENTER);
        delRowColPanel.setBorder(title);
        JLabel delRowLabel = new JLabel("Delete Row(s): ");
        JLabel delColLabel = new JLabel("Delete Column(s): ");
        delRowField = new JTextField(10);
        delColField = new JTextField(10);
        delRowField.addFocusListener(this);
        delColField.addFocusListener(this);
        
        c.insets = new Insets(0, 5, 0, 5);
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 0;
        delRowColPanel.add(delRowLabel, c);
        
        c.gridx = 1;
        c.weightx = 1;
        delRowColPanel.add(delRowField,c);
        
        c.gridx = 2;
        c.weightx = 0;
        delRowColPanel.add(delColLabel, c);
        
        c.gridx = 3;
        c.weightx = 1;
        delRowColPanel.add(delColField, c);
        
        c.gridx = 4;
        c.weightx = 0;
        delRowColPanel.add(new HelpButton(Help.DELETE_ROW_COLUMN));
        
        return delRowColPanel;
    }
    
    /**
     * Creates the panel which displays the information about the data (features, examples, classes).
     * @return The panel which displays the information about the data.
     */
    private JPanel createCountsPanel() {
        JPanel countsPanel = new JPanel(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        Border blackline = BorderFactory.createLineBorder(Color.black);
        TitledBorder title = BorderFactory.createTitledBorder(blackline,
        "Data Observations");
        title.setTitleJustification(TitledBorder.CENTER);
        countsPanel.setBorder(title);
        featureCount = new JLabel("No Data Loaded");
        dataCount = new JLabel("No Data Loaded");
        classesCount = new JLabel("No Data Loaded");
        c.gridx = 0;
        c.gridy = 0;
        c.weightx = 1;
        countsPanel.add(dataCount, c);
        c.gridx = 1;
        countsPanel.add(featureCount, c);
        c.gridx = 2;
        countsPanel.add(classesCount, c);
        c.gridx = 3;
        c.weightx = 0;
        countsPanel.add(new HelpButton(Help.OBSERVATIONS));
        return countsPanel;
    }
    
    /**
     * Creates the panel with the buttons for display the entire data and the classification key.
     * @return The panel with the buttons for display the entire data and the classification key.
     */
    private JPanel createButtons() {
        JPanel p = new JPanel(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        
        viewKeyButton = new JButton(" View Class Label Key ");
        //viewKeyButton.setBorder(BorderFactory.createRaisedBevelBorder());
        viewKeyButton.addActionListener(this);
        
        viewAllButton = new JButton("      View All Data      ");
        //viewAllButton.setBorder(BorderFactory.createRaisedBevelBorder());
        viewAllButton.addActionListener(this);
        
        c.insets = new Insets(5, 50, 5, 50);
        c.weightx = 0;
        p.add(viewAllButton, c);
        c.gridx = 1;
        p.add(viewKeyButton, c);
        return p;
    }
    
    /**
     * Creates the panel with the option for what portion of the data needs to be viewed.
     * @return The panel with the option for what portion of the data needs to be viewed.
     */
    private JPanel createViewOptions() {
        JPanel p = new JPanel(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        
        viewOptionsLabels = new JLabel[numViewOptions];
        viewOptions = new JTextField[numViewOptions];
        for (int i = 0; i < numViewOptions; ++i) {
            viewOptionsLabels[i] = new JLabel(viewOptionsTitles[i]);
            viewOptions[i] = new JTextField("1", 4);
            viewOptions[i].addFocusListener(this);
            c.insets = new Insets(0, 3, 0, 3);
            c.gridx = i*2;
            c.gridy = 0;
            c.weightx = 0;
            p.add(viewOptionsLabels[i], c);
            c.gridx = i*2+1;
            c.weightx = 1;
            p.add(viewOptions[i], c);
        }
        viewOptions[0].setText("" + rowsToView);
        viewOptions[2].setText("" + columnsToView);
        c.gridx = 8;
        c.weightx = 0;
        p.add(new HelpButton(Help.DATA_WINDOW));
        p.validate();
        return p;
    }
    
    public void focusGained(FocusEvent e) {
        if (e.getSource() == delRowField)
            delRowField.selectAll();
        else if (e.getSource() == delColField)
            delColField.selectAll();
        else if (e.getSource() == viewOptions[0])
            viewOptions[0].selectAll();
        else if (e.getSource() == viewOptions[1])
            viewOptions[1].selectAll();
        else if (e.getSource() == viewOptions[2])
            viewOptions[2].selectAll();
        else if (e.getSource() == viewOptions[3])
            viewOptions[3].selectAll();
    }
    
    /**
     * Parses the number given in the "display row/column" options.
     * @param oldValue The old value of the box.
     * @param maxValue The maximum value of the box
     * @param box The box.
     * @return The parsed number of the box.
     */
    private int process(int oldValue, int maxValue, JTextField box) {
        try {
            int newValue = Integer.parseInt(box.getText());
            if (newValue <= 0) {
                oldValue = 1;
                throw new NumberFormatException("I am lazy");
            }
            if (newValue > maxValue) {
                oldValue = maxValue;
                throw new NumberFormatException("Yes I am");
            }
            return newValue;
        } catch (NumberFormatException e) {
            box.setText(Integer.toString(oldValue));
            return oldValue;
        }
    }
    
    public void focusLost(FocusEvent e) {
        
        Object source = e.getSource();
        
        if (source == viewOptions[0])
            rowsToView = process(rowsToView, currData.numRows(), viewOptions[0]);
        if (source == viewOptions[2])
            columnsToView = process(columnsToView, currData.numCols(), viewOptions[2]);
        if (source == viewOptions[3])
            startColVal = process(startColVal, currData.numCols(), viewOptions[3]);
        if (source == viewOptions[1])
            startRowVal = process(startRowVal, currData.numRows(), viewOptions[1]);
        
        if (source == delRowField || source == delColField) {
            try {
                rowDel = new ArrayList();
                String[] args = delRowField.getText().split("[, ]");
                for (int i = 0; i < args.length; ++i) 
                    if (!args[i].equals("")) {
                        int d = Integer.parseInt(args[i]) - 1;
                        if (d < 0)
                            delRowField.setText(Messages.ERROR_MUST_BE_POSITIVE);
                        else
                            rowDel.add(new Integer(d));
                    }
            } catch (NumberFormatException ex) {
                delRowField.setText(Messages.ERROR_COMMA_LIST);
            }
            try {
                colDel = new ArrayList();
                String[] args = delColField.getText().split("[, ]");
                for (int i = 0; i < args.length; ++i) 
                    if (!args[i].equals("")) {
                        int d = Integer.parseInt(args[i]) - 1;
                        if (d < 0)
                            delColField.setText(Messages.ERROR_MUST_BE_POSITIVE);
                        else
                            colDel.add(new Integer(d));
                    }
            } catch (NumberFormatException ex) {
                delColField.setText(Messages.ERROR_COMMA_LIST);
            }
        }
        updateModifyProperty();
        loadTable(originalData, originalClasses, modifyProp);
        updateCounts();
    }
    
    /**
     * updates the counts
     */
    private void updateCounts() {
        featureCount.setText("Features: " + currData.numCols());
        dataCount.setText("Examples: " + currData.numRows());
    }
    
    /**
     * updates the ModifyProperty with the values currently selected in the GUI
     */
    private void updateModifyProperty() {
    	try {
            rowDel = new ArrayList();
            String[] args = delRowField.getText().split("[, ]");
            for (int i = 0; i < args.length; ++i) 
                if (!args[i].equals("")) {
                    int d = Integer.parseInt(args[i]) - 1;
                    if (d < 0) {
                        //delRowField.setText(Messages.ERROR_MUST_BE_POSITIVE);
                    }
                    else
                        rowDel.add(new Integer(d));
                }
        } catch (NumberFormatException ex) {
            //delRowField.setText(Messages.ERROR_COMMA_LIST);
        }
        try {
            colDel = new ArrayList();
            String[] args = delColField.getText().split("[, ]");
            for (int i = 0; i < args.length; ++i) 
                if (!args[i].equals("")) {
                    int d = Integer.parseInt(args[i]) - 1;
                    if (d < 0) {
                        //delColField.setText(Messages.ERROR_MUST_BE_POSITIVE);
                    }
                    else
                        colDel.add(new Integer(d));
                }
        } catch (NumberFormatException ex) {
            //delColField.setText(Messages.ERROR_COMMA_LIST);
        }
        modifyProp = new ModifyProperty();
        modifyProp.getDeletedColumns().addAll(colDel);
        modifyProp.getDeletedRows().addAll(rowDel);
    }
    
    /**
     * updates and returns the ModifyProperty
     * @return a ModifyProperty
     */
    public ModifyProperty getModifyProperty() {
        updateModifyProperty();
        return modifyProp;
    }
    
    /**
     * returns the data being stored by the data panel
     * @return the current Data
     */
    public IMatrix getData() {
        return Modifier.modifyData(originalData, getModifyProperty());
    }
    
    /**
     * returns the classifications being stored by the data panel
     * @return the current classifcations
     */
    public IMatrix getClassifications() {
    	if (originalClasses == null) return null;
    	IMatrix temp =Modifier.modifyClasses(originalClasses, getModifyProperty());
        return temp;
    }
    
    /**
     * Loads in pre-set options.
     * @param rowsToDelete The rows to be deleted.
     * @param colsToDelete The columns to be deleted.
     */
    public void setOptions(TreeSet rowsToDelete, TreeSet colsToDelete) {
        String s = "";
        ArrayList rtd = new ArrayList();
        rtd.addAll(rowsToDelete);
        for (int i = 0; i < rowsToDelete.size(); ++i) {
            s += rtd.get(i);
            if (i != rtd.size()-1) s += ", ";
        }
        delRowField.setText(s);
        
        s = "";
        ArrayList ctd = new ArrayList();
        ctd.addAll(colsToDelete);
        for (int i = 0; i < colsToDelete.size(); ++i) {
            s += ctd.get(i);
            if (i != ctd.size()-1) s += ", ";
        }
        delColField.setText(s);
    }
    
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == viewKeyButton) {
            if (map != null) showKey();
            else {
            	JOptionPane.showMessageDialog(panel,
                        Messages.ERROR_NO_KEY,
                        Messages.ERROR_NO_KEY_TITLE, JOptionPane.INFORMATION_MESSAGE);
            }
        }
        if (e.getSource() == viewAllButton)
        {
            rowsToView = currData.numRows(); viewOptions[0].setText(""+rowsToView);
            columnsToView = currData.numCols(); viewOptions[2].setText(""+columnsToView);
            startColVal = 1; viewOptions[3].setText(""+(startColVal));
            startRowVal = 1; viewOptions[1].setText(""+(startRowVal));
            loadTable(null);
        }
    }
}
