package edu.duke.cs.smlr.gui.panels;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

import edu.duke.cs.smlr.Graphical;
import edu.duke.cs.smlr.gui.help.Help;
import edu.duke.cs.smlr.gui.help.HelpButton;
import edu.duke.cs.smlr.util.Constants;
import edu.duke.cs.smlr.util.Defaults;

/**Creates a panel used to choose the overall learning task, set filenames, and
 * choose primary parameters (k for k-fold)
 * 
 * @author Jason Bosko, Matt Edwards
 */
public class TaskPanel extends AbstractPanel implements ItemListener, ActionListener, FocusListener {
	
	private JPanel trainPanel;
	private JPanel evaluationPanel;
	private JPanel usePanel;
	private JPanel loadClassifierPanel;
	private JPanel saveClassifierPanel;
	
	private JRadioButton evalButton;
	private JRadioButton useButton;
	private JRadioButton loadButton;
	private JRadioButton testing;
	private JRadioButton loocv;
	private JRadioButton kfold;
	private JRadioButton rbClassChoices[];	
	
	private JTextField trainingField;
	private JTextField testField;
	private JTextField unlabeledField;
	private JTextField classifierField;
	private JTextField originalField;
	private JTextField saveClassifierField;
	
	private JButton trainingButton;
	private JButton testButton;
	private JButton unlabeledButton;
	private JButton classifierButton;
	private JButton originalButton;
	private JButton saveClassifierButton;
	
	private JTextField kField;	
	
	private final String classChoicesTitles[] = { "Guess", "First", "Last", "Choose: " };
	private final int numClassChoices = 4;
	private JTextField columnChoice;
	private JComboBox kfoldRandom;
	
	private File trainFile;
	private File testFile;
	private File unlabeledFile;
	private File classifierFile;
	private File originalFile;
	private File saveClassifierFile;
	
	/**
	 * Constructs a task panel for the GUI.
	 * @param parent The GUI.
	 */
	public TaskPanel(Graphical parent) {
		super(parent);
		this.parent = parent;
		trainFile = new File(Constants.NO_FILE_LOADED);
		testFile = new File(Constants.NO_FILE_LOADED);
		unlabeledFile = new File(Constants.NO_FILE_LOADED);
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
		panel.add(createTaskPanel());
		panel.add(createTrainingDataPanel());
		panel.add(createLoadClassifierPanel());
		panel.add(createEvaluationPanel());
		panel.add(createUsePanel());
		panel.add(createSaveClassifierPanel());
	}
	
	/**
	 * Creates the panel that specifies the overall task to complete.
	 * @return The panel that specifies the overall task to complete.
	 */
	private JPanel createTaskPanel() {
		JPanel p = new JPanel(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		Border blackline = BorderFactory.createLineBorder(Color.black);
		TitledBorder title = BorderFactory.createTitledBorder(blackline, "Task");
		title.setTitleJustification(TitledBorder.CENTER);
		p.setBorder(title);
		
		evalButton = new JRadioButton("Learn & Evaluate Classifier", true);
		useButton  = new JRadioButton("Learn & Use Classifier", false);
		loadButton = new JRadioButton("Load & Use a Saved Classifier", false);
		evalButton.addItemListener(this);
		useButton.addItemListener(this);
		loadButton.addItemListener(this);
		
		ButtonGroup bg = new ButtonGroup();
		bg.add(evalButton);
		bg.add(useButton);
		//bg.add(loadButton);
		
		c.weightx = 1;
		p.add(evalButton, c);
		
		c.gridx = 1;
		p.add(useButton, c);
		
		//c.gridx = 2;
		//p.add(loadButton, c);
		
		c.gridx = 2;
		c.weightx = 0;
		p.add(new HelpButton(Help.TASK), c);
		return p;
	}
	
	/**
	 * Creates the panel with the options associated with loading the training data.
	 * @return The panel with the options associated with loading the training data.
	 */
	private JPanel createTrainingDataPanel() {
		trainPanel = new JPanel(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		Border blackline = BorderFactory.createLineBorder(Color.black);
		TitledBorder title = BorderFactory.createTitledBorder(blackline, "Training Data");
		title.setTitleJustification(TitledBorder.CENTER);
		trainPanel.setBorder(title);
		
		trainingField = new JTextField();
		trainingField.addFocusListener(this);
		trainingButton = new JButton(" Browse ");
		//trainingButton.setBorder(BorderFactory.createRaisedBevelBorder());
		trainingButton.addActionListener(this);
		
		trainPanel.add(new JLabel("Training Data: "), c);
		c.gridx = 1;
		c.weightx = 1;
		trainPanel.add(trainingField, c);
		c.gridx = 2;
		c.weightx = 0;
		c.insets = new Insets(5, 5, 5, 5);
		trainPanel.add(trainingButton, c);
		c.insets = new Insets(0,0,0,0);
		c.gridx = 3;
		trainPanel.add(new HelpButton(Help.TRAINING_DATA), c);
		
		JPanel rbPanel = new JPanel();
		rbPanel.setLayout(new BoxLayout(rbPanel, BoxLayout.X_AXIS));
		rbPanel.add(new JLabel("Column With Class Labels: "));
		
		ButtonGroup bg = new ButtonGroup();
		rbClassChoices = new JRadioButton[numClassChoices];
		for (int i = 0; i < numClassChoices; ++i) {
			rbClassChoices[i] = new JRadioButton(classChoicesTitles[i], false);
			rbClassChoices[i].addActionListener(new ActionListener() {
				//this is pretty hacky, we should organize this class a lot better
				//this makes any changes to feature-column force a reload
				public void actionPerformed(ActionEvent e) {
					((Graphical) parent).newData = true;
					((Graphical) parent).newTestData = true;
				}				
			});
			bg.add(rbClassChoices[i]);
			rbPanel.add(rbClassChoices[i]);
		}
		
		rbClassChoices[0].setSelected(Defaults.CLASS_LABELS == Constants.READER_GUESS);
		rbClassChoices[1].setSelected(Defaults.CLASS_LABELS == Constants.READER_FIRST_COLUMN);
		rbClassChoices[2].setSelected(Defaults.CLASS_LABELS == Constants.READER_LAST_COLUMN);
		
		rbClassChoices[3].addItemListener(this);
		columnChoice = new JTextField("", 4);
		columnChoice.setEditable(false);
		columnChoice.setBackground(trainPanel.getBackground());		
		columnChoice.addFocusListener(new FocusListener() {
			public void focusGained(FocusEvent e) {				
			}

			public void focusLost(FocusEvent e) {
				((Graphical) parent).newData = true;		
				((Graphical) parent).newTestData = true;	
			}			
		});
		
		rbPanel.add(columnChoice);
		
		c.gridx = 0; c.gridy = 1;
		c.weightx = 1; c.gridwidth = GridBagConstraints.REMAINDER;
		trainPanel.add(rbPanel, c);
				
		return trainPanel;
	}
	
	/**
	 * Creates the panel for choosing options associated with evaluating a classifier.
	 * @return The panel for choosing options associated with evaluating a classifier.
	 */
	private JPanel createEvaluationPanel() {
		evaluationPanel = new JPanel(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		Border blackline = BorderFactory.createLineBorder(Color.black);
		TitledBorder title = BorderFactory.createTitledBorder(blackline, "Evaluation Options");
		title.setTitleJustification(TitledBorder.CENTER);
		evaluationPanel.setBorder(title);
		
		testing = new JRadioButton("Use Test Data: ", Defaults.BEHAVIOR == Constants.TEST_TESTING_DATA);
		loocv   = new JRadioButton("Leave-One-Out Cross Validation", Defaults.BEHAVIOR == Constants.TEST_LOOCV);
		kfold   = new JRadioButton("K-Fold Cross Validation", Defaults.BEHAVIOR == Constants.TEST_KFOLD);
		ButtonGroup bg = new ButtonGroup();
		bg.add(testing); bg.add(loocv); bg.add(kfold);
		testing.addItemListener(this);
		loocv.addItemListener(this);
		kfold.addItemListener(this);
		
		testField = new JTextField();
		testField.setEditable(testing.isSelected());
		testField.setBackground(Color.WHITE);
		testField.addFocusListener(this);
		if (!testing.isSelected()) testField.setBackground(panel.getBackground());
		testButton = new JButton(" Browse ");
		//testButton.setBorder(BorderFactory.createRaisedBevelBorder());
		testButton.setEnabled(testing.isSelected());
		testButton.addActionListener(this);
		
		kField = new JTextField();
		kField.setEditable(testing.isSelected());
		if (!kfold.isSelected()) kField.setBackground(panel.getBackground());
		kField.addFocusListener(this);
		String choices[] = {"Within Classes","Across Classes","None"};
		kfoldRandom = new JComboBox(choices);
		if (Defaults.RANDOMIZATION == Constants.KFOLD_RATIO_RANDOM) kfoldRandom.setSelectedIndex(0);
		if (Defaults.RANDOMIZATION == Constants.KFOLD_PURE_RANDOM) kfoldRandom.setSelectedIndex(1);
		if (Defaults.RANDOMIZATION == Constants.KFOLD_NON_RANDOM) kfoldRandom.setSelectedIndex(2);
		kfoldRandom.setEnabled(kfold.isSelected());
		if (!kfold.isSelected()) testField.setBackground(panel.getBackground());
		
		JPanel testPanel = new JPanel(new GridBagLayout());
		GridBagConstraints C = new GridBagConstraints();
		C.fill = GridBagConstraints.HORIZONTAL;
		C.gridx = 0;
		C.gridy = 0;
		testPanel.add(testing, C);
		C.insets = new Insets(0,0,0,10);
		C.gridx = 1;
		C.weightx = 1;
		testPanel.add(testField, C);
		C.gridx = 2;
		C.weightx = 0;
		testPanel.add(testButton, C);
		
		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		evaluationPanel.add(testPanel, c);
		c.gridx = 1;
		c.weightx = 0;
		evaluationPanel.add(new HelpButton(Help.EVALUATION_OPTIONS), c);
		
		c.gridx = 0;
		c.gridy = 1;
		c.weightx = 1;
		evaluationPanel.add(loocv, c);
		
		JPanel kfoldPanel = new JPanel(new GridBagLayout());
		C = new GridBagConstraints();
		C.fill = GridBagConstraints.HORIZONTAL;
		C.gridx = 0;
		C.gridy = 0;
		C.gridwidth = GridBagConstraints.REMAINDER;
		kfoldPanel.add(kfold, C);
		C.insets = new Insets(0,5,0,5);
		C.gridx = 0;
		C.gridy = 1;
		C.gridwidth = 1;
		kfoldPanel.add(new JLabel("Enter K:"), C);
		C.gridx = 1;
		C.weightx = 1;
		kfoldPanel.add(kField, C);
		C.gridx = 2;
		C.weightx = 0;
		kfoldPanel.add(new JLabel("Randomization:"), C);
		C.gridx = 3;
		kfoldPanel.add(kfoldRandom, C);
		
		c.gridx = 0;
		c.gridy = 2;
		evaluationPanel.add(kfoldPanel, c);
		
		evaluationPanel.setVisible(evalButton.isSelected());
		return evaluationPanel;
	}
	
	/**
	 * Creates the panel for options associated with using a classifier on unlabeled data.
	 * @return The panel for options associated with using a classifier on unlabeled data.
	 */
	private JPanel createUsePanel() {
		usePanel = new JPanel(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		Border blackline = BorderFactory.createLineBorder(Color.black);
		TitledBorder title = BorderFactory.createTitledBorder(blackline, "Use Options");
		title.setTitleJustification(TitledBorder.CENTER);
		usePanel.setBorder(title);
		
		unlabeledField = new JTextField();
		unlabeledField.addFocusListener(this);
		unlabeledButton = new JButton(" Browse ");
		//unlabeledButton.setBorder(BorderFactory.createRaisedBevelBorder());
		unlabeledButton.addActionListener(this);
		
		usePanel.add(new JLabel("Unlabeled Data: "), c);
		c.gridx = 1;
		c.weightx = 1;
		c.insets = new Insets(0, 5, 0, 5);
		usePanel.add(unlabeledField, c);
		c.gridx = 2;
		c.weightx = 0;
		usePanel.add(unlabeledButton, c);
		c.gridx = 3;
		usePanel.add(new HelpButton(Help.USE_OPTIONS), c);
		
		usePanel.setVisible(useButton.isSelected());
		return usePanel;
	}
	
	/**
	 * Creates the panel for options associated with loading a classifier.
	 * This option is not included in this release (1.0.1)
	 * @return The panel for options associated with loading a classifier.
	 */
	private JPanel createLoadClassifierPanel() {
		loadClassifierPanel = new JPanel(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		Border blackline = BorderFactory.createLineBorder(Color.black);
		TitledBorder title = BorderFactory.createTitledBorder(blackline, "Load a Classifier");
		title.setTitleJustification(TitledBorder.CENTER);
		loadClassifierPanel.setBorder(title);
		
		classifierField = new JTextField();
		classifierField.addFocusListener(this);
		classifierButton = new JButton(" Browse ");
		classifierButton.addActionListener(this);
		
		originalField = new JTextField();
		originalField.addFocusListener(this);
		originalButton = new JButton(" Browse ");
		originalButton.addActionListener(this);
		
		loadClassifierPanel.add(new JLabel("Classifier File: "), c);
		c.gridx = 1;
		c.weightx = 1;
		c.insets = new Insets(5, 5, 5, 5);
		loadClassifierPanel.add(classifierField, c);
		c.gridx = 2;
		c.weightx = 0;
		loadClassifierPanel.add(classifierButton, c);
		c.gridx = 3;
		loadClassifierPanel.add(new HelpButton(Help.USE_OPTIONS), c);
		
		c.gridy = 1;
		c.gridx = 0;
		c.insets = new Insets(0,0,0,0);
		loadClassifierPanel.add(new JLabel("Orignal Training File: "), c);
		c.gridx = 1;
		c.weightx = 1;
		c.insets = new Insets(5, 5, 5, 5);
		loadClassifierPanel.add(originalField, c);
		c.gridx = 2;
		c.weightx = 0;
		loadClassifierPanel.add(originalButton, c);
		c.gridx = 3;
		loadClassifierPanel.add(new HelpButton(Help.USE_OPTIONS), c);
		
		loadClassifierPanel.setVisible(loadButton.isSelected());
		return loadClassifierPanel;
	}
	
	/**
	 * Creates the panel for options associated with saving a classifier.
	 * This option is not included in this release (1.0.1)
	 * @return The panel for options associated with saving a classifier.
	 */
	private JPanel createSaveClassifierPanel() {
		saveClassifierPanel = new JPanel(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.fill = GridBagConstraints.HORIZONTAL;
		Border blackline = BorderFactory.createLineBorder(Color.black);
		TitledBorder title = BorderFactory.createTitledBorder(blackline, "Save the Classifier");
		title.setTitleJustification(TitledBorder.CENTER);
		saveClassifierPanel.setBorder(title);
		
		saveClassifierField = new JTextField();
		saveClassifierField.addFocusListener(this);
		saveClassifierButton = new JButton(" Browse ");
		//unlabeledButton.setBorder(BorderFactory.createRaisedBevelBorder());
		saveClassifierButton.addActionListener(this);
		
		saveClassifierPanel.add(new JLabel("Save a Classifier File: "), c);
		c.gridx = 1;
		c.weightx = 1;
		c.insets = new Insets(0, 5, 0, 5);
		saveClassifierPanel.add(saveClassifierField, c);
		c.gridx = 2;
		c.weightx = 0;
		saveClassifierPanel.add(saveClassifierButton, c);
		c.gridx = 3;
		saveClassifierPanel.add(new HelpButton(Help.USE_OPTIONS), c);
		
		saveClassifierPanel.setVisible(useButton.isSelected());
		return saveClassifierPanel;
	}

	public void itemStateChanged(ItemEvent e) {
		((Graphical) parent).setKFoldDisplay(loocv.isSelected() || kfold.isSelected());
		Object source = e.getSource();
		boolean onoff = e.getStateChange() == ItemEvent.SELECTED;
		if (source == evalButton && onoff) {
			trainPanel.setVisible(true);
			usePanel.setVisible(false);
			evaluationPanel.setVisible(true);
			loadClassifierPanel.setVisible(false);
			saveClassifierPanel.setVisible(false);
		}
		if (source == useButton && onoff) {
			trainPanel.setVisible(true);
			evaluationPanel.setVisible(false);
			usePanel.setVisible(true);
			loadClassifierPanel.setVisible(false);
			//saveClassifierPanel.setVisible(true);
		}
		if (source == loadButton && onoff) {
			trainPanel.setVisible(false);
			evaluationPanel.setVisible(false);
			loadClassifierPanel.setVisible(true);
			usePanel.setVisible(true);
			saveClassifierPanel.setVisible(false);
		}
		if (source == rbClassChoices[3]) {
			if (rbClassChoices[3].isSelected()) {
				columnChoice.setEditable(true);
				columnChoice.setText("1");
				columnChoice.setBackground(Color.WHITE);
			} else {
				columnChoice.setEditable(false);
				columnChoice.setText("");
				columnChoice.setBackground(panel.getBackground());
			}
		}
		if (source == testing) {
			if (onoff)
				testField.setBackground(Color.WHITE);
			else
				testField.setBackground(panel.getBackground());
			testField.setEditable(onoff);
			testButton.setEnabled(onoff);
		}
		if (source == kfold) {
			if (onoff)
				kField.setBackground(Color.WHITE);
			else
				kField.setBackground(panel.getBackground());
			kField.setEditable(onoff);
			kfoldRandom.setEnabled(onoff);
		}
	}

	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		if (source == trainingButton) {
        	JFileChooser fc = edu.duke.cs.smlr.util.Utilities.newFileChooser();
			if (fc.showOpenDialog(panel) == JFileChooser.APPROVE_OPTION) {
				((Graphical) parent).newData = true;
			    trainFile = fc.getSelectedFile();
				trainingField.setText(trainFile.getPath());
			}
			edu.duke.cs.smlr.util.Utilities.setDirectory(fc.getCurrentDirectory());			
		}
		if (source == testButton) {
        	JFileChooser fc = edu.duke.cs.smlr.util.Utilities.newFileChooser();
			if (fc.showOpenDialog(panel) == JFileChooser.APPROVE_OPTION) {
				((Graphical) parent).newTestData = true;
				testFile = fc.getSelectedFile();
				testField.setText(testFile.getPath());
			}
			edu.duke.cs.smlr.util.Utilities.setDirectory(fc.getCurrentDirectory());			
		}
		if (source == unlabeledButton) {
        	JFileChooser fc = edu.duke.cs.smlr.util.Utilities.newFileChooser();
			if (fc.showOpenDialog(panel) == JFileChooser.APPROVE_OPTION) {
				((Graphical) parent).newUnlabeledData = true;
				unlabeledFile = fc.getSelectedFile();
				unlabeledField.setText(unlabeledFile.getPath());
			}
			edu.duke.cs.smlr.util.Utilities.setDirectory(fc.getCurrentDirectory());			
		}
		if (source == classifierButton) {
        	JFileChooser fc = edu.duke.cs.smlr.util.Utilities.newFileChooser();
			if (fc.showOpenDialog(panel) == JFileChooser.APPROVE_OPTION) {
				classifierFile = fc.getSelectedFile();
				classifierField.setText(classifierFile.getPath());
			}
			edu.duke.cs.smlr.util.Utilities.setDirectory(fc.getCurrentDirectory());			
		}
		if (source == originalButton) {
        	JFileChooser fc = edu.duke.cs.smlr.util.Utilities.newFileChooser();
			if (fc.showOpenDialog(panel) == JFileChooser.APPROVE_OPTION) {
				originalFile = fc.getSelectedFile();
				originalField.setText(originalFile.getPath());
			}
			edu.duke.cs.smlr.util.Utilities.setDirectory(fc.getCurrentDirectory());			
		}
		if (source == saveClassifierButton) {
        	JFileChooser fc = edu.duke.cs.smlr.util.Utilities.newFileChooser();
			if (fc.showOpenDialog(panel) == JFileChooser.APPROVE_OPTION) {
				saveClassifierFile = fc.getSelectedFile();
				saveClassifierField.setText(saveClassifierFile.getPath());
			}
			edu.duke.cs.smlr.util.Utilities.setDirectory(fc.getCurrentDirectory());			
		}
	}

	public void focusGained(FocusEvent e) {
		JTextField source = (JTextField) e.getSource();
		source.selectAll();
	}

	public void focusLost(FocusEvent e) {
		JTextField source = (JTextField) e.getSource();
		if (source == trainingField) {
			trainFile = new File(source.getText());
			((Graphical) parent).newData = true;
		}
		else if (source == testField) {
			testFile = new File(source.getText());
			((Graphical) parent).newTestData = true;
		}
		else if (source == unlabeledField) {
			unlabeledFile = new File(source.getText());
			((Graphical) parent).newUnlabeledData = true;
		}
		else if (source == classifierField) {
			classifierFile = new File(source.getText());
		}
		else if (source == originalField) {
			originalFile = new File(source.getText());
		}
		else if (source == saveClassifierField) {
			saveClassifierFile = new File(source.getText());
		}
		else try {
			int n = Integer.parseInt(source.getText());
			if (n <= 0)
				source.setText("Must be positive");
		}
		catch (NumberFormatException er) {
			source.setText("Incorrect format");
		}
	}

	/**
	 * @return the selected training data filename
	 */
	public String getTrainingFile() {
		return trainFile.getPath();
	}
	
	/**
	 * @return the selected testing data filename
	 */
	public String getTestFile() {
		return testFile.getPath();
	}
	
	/**
	 * @return the selected unlabeled test data filename
	 */
	public String getUnlabeledFile() {
		return unlabeledFile.getPath();
	}
	
	/**
	 * @return the selected original training data filename
	 */
	public String getClassifierFile() {
		return classifierFile.getPath();
	}
	
	/**
	 * @return the selected original training data filename
	 */
	public String getOriginalFile() {
		return originalFile.getPath();
	}
	
	/**
	 * @return the selected original training data filename
	 */
	public String getSaveClassifierFile() {
		return saveClassifierFile.getPath();
	}
	
	/**
	 * @return whether training data has been loaded
	 */
	public boolean isTrainingLoaded() {
		return !trainFile.getPath().equals(Constants.NO_FILE_LOADED);
	}
	
	/**
	 * @return whether testing data has been loaded
	 */
	public boolean isTestLoaded() {
		return !testFile.getPath().equals(Constants.NO_FILE_LOADED);
	}
	
	/**
	 * @return whether unlabeled data has been loaded
	 */
	public boolean isUnlabeledLoaded() {
		return !unlabeledFile.getPath().equals(Constants.NO_FILE_LOADED);
	}
	
	/**
	 * @return the selected column for classifications
	 *  (should be passed to adaptive reader)
	 */
	public int getClassColumn() {
		int classChoice = -1;
		if (rbClassChoices[0].isSelected())
			classChoice = Constants.READER_GUESS;
		if (rbClassChoices[1].isSelected())
			classChoice = Constants.READER_FIRST_COLUMN;
		if (rbClassChoices[2].isSelected())
			classChoice = Constants.READER_LAST_COLUMN;
		if (rbClassChoices[3].isSelected())
			try {
				classChoice = Integer.parseInt(columnChoice.getText()) - 1;
			} catch (NumberFormatException e) {
				columnChoice.setText("");
				classChoice = Constants.READER_GUESS;
			}
		return classChoice;
	}
	
	/**
	 * @return the selected learning task
	 */
	public int getTestChoice() {
		if (evalButton.isSelected() && testing.isSelected())
			return Constants.TEST_TESTING_DATA;
		if (evalButton.isSelected() && kfold.isSelected())
			return Constants.TEST_KFOLD;
		if (evalButton.isSelected() && loocv.isSelected())
			return Constants.TEST_LOOCV;
		if (useButton.isSelected())
			return Constants.TEST_UNLABELED;
		return Constants.TEST_NONE;
	}
	
	/**
	 * @return the selected K
	 */
	public int getK() {
		try {
			return Integer.parseInt(kField.getText());
		} catch (NumberFormatException e) {
			return Defaults.k;
		}
	}
	
	/**
	 * @return the selected cross-validation randomization behavior
	 */
	public int getKfoldOption() {
		if (kfoldRandom.getSelectedItem().equals("Within Classes"))
			return Constants.KFOLD_RATIO_RANDOM;
		if (kfoldRandom.getSelectedItem().equals("Across Classes"))
			return Constants.KFOLD_PURE_RANDOM;
		if (kfoldRandom.getSelectedItem().equals("None"))
			return Constants.KFOLD_NON_RANDOM;
		return Constants.KFOLD_PURE_RANDOM;
	}

	/**
	 * Sets the task panel with options loaded from a settings file
	 * @param task The task to be done
	 * @param trainingFile The file with the training data
	 * @param classChoice The choice for the label column
	 * @param testFile The file with the test data
	 * @param k The parameter k for the k-fold cross-validation
	 * @param behavior The randomization behavior for k-fold cross-validation
	 */
	public void setOptions(int task, String trainingFile, int classChoice, String testFile, String unlabeledFile, int k, int behavior) {
		if (task == Constants.TEST_UNLABELED) {
			useButton.setSelected(true);
			unlabeledField.setText(unlabeledFile);
			this.unlabeledFile = new File(unlabeledFile);
		}
		if (task == Constants.TEST_TESTING_DATA) {
			evalButton.setSelected(true);
			testing.setSelected(true);
			testField.setText(testFile);
			this.testFile = new File(testFile);
		}
		if (task == Constants.TEST_LOOCV) {
			evalButton.setSelected(true);
			loocv.setSelected(true);
		}
		if (task == Constants.TEST_KFOLD) {
			evalButton.setSelected(true);
			kfold.setSelected(true);
			kField.setText(""+k);
			if (behavior == Constants.KFOLD_NON_RANDOM) {
				kfoldRandom.removeAllItems();
				kfoldRandom.addItem("None");
				kfoldRandom.addItem("Within Classes");
				kfoldRandom.addItem("Across Classes");
			}
			if (behavior == Constants.KFOLD_PURE_RANDOM) {
				kfoldRandom.removeAllItems();
				kfoldRandom.addItem("Across Classes");
				kfoldRandom.addItem("Within Classes");
				kfoldRandom.addItem("None");
			}
			if (behavior == Constants.KFOLD_RATIO_RANDOM) {
				kfoldRandom.removeAllItems();
				kfoldRandom.addItem("Within Classes");
				kfoldRandom.addItem("Across Classes");
				kfoldRandom.addItem("None");
			}
		}
		if (!trainingFile.equals(Constants.NO_FILE_LOADED)) { 
			trainingField.setText(trainingFile);
			trainFile = new File(trainingFile);
		}
		
		if (classChoice == Constants.READER_GUESS)
			rbClassChoices[0].setSelected(true);
		else if (classChoice == Constants.READER_FIRST_COLUMN)
			rbClassChoices[1].setSelected(true);
		else if (classChoice == Constants.READER_LAST_COLUMN)
			rbClassChoices[2].setSelected(true);
		else {
			rbClassChoices[3].setSelected(true);
			columnChoice.setText(""+classChoice);
		}
	}
}
