package edu.duke.cs.smlr.gui.table;

import java.awt.Color;
import java.awt.Component;

import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.table.DefaultTableCellRenderer;

/**A renderer for use with tables that alternates row backgrounds between white
 * and light gray.  Extends the DefaultTableCellRenderer so most other behavior
 * is "normal."
 * 
 * @author Matt Edwards
 */

public class AlternatingRowRenderer extends DefaultTableCellRenderer {
    
    public AlternatingRowRenderer () {
        super();
        this.setOpaque(true);
    }
    
    public void setAlternatingBackground(Component c, JTable table, boolean isSelected, boolean hasFocus, int row, int column) {
        if (isSelected) {
            c.setForeground(table.getSelectionForeground());
            c.setBackground(table.getSelectionBackground());
        }
        else {
            c.setBackground(row % 2 == 1 ? new Color(235, 235, 235) : Color.white);
        }
        
        setFont(table.getFont());
        if (hasFocus) {
            setBorder( UIManager.getBorder("Table.focusCellHighlightBorder") );
            if (!isSelected && table.isCellEditable(row, column)) {
                Color col;
                col = UIManager.getColor("Table.focusCellForeground");
                if (col != null) {
                    c.setForeground(col);
                }
                col = UIManager.getColor("Table.focusCellBackground");
                if (col != null) {
                    c.setBackground(col);
                }
            }
        } else {
            setBorder(noFocusBorder);
        }        
    }
    
    /*
     * (non-Javadoc)
     * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
     */
    public Component getTableCellRendererComponent(JTable table, Object value,
            boolean isSelected, boolean hasFocus, int row, int column) {
        
        setAlternatingBackground(this, table, isSelected, hasFocus, row, column);
        
        return super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
    }
    
}
