package edu.duke.cs.smlr.gui.table;

import java.awt.Color;
import java.awt.Component;

import javax.swing.JTable;

/**A renderer for use with tables that renders rows according to the value in their last
 * column.
 * <br>
 * This renderer extends from AlternatingRowRenderer so it alternates the background
 * colors of rows it renders.
 * 
 * @author Matt Edwards
 */

public class ColoredRenderer extends AlternatingRowRenderer {
    
    public static final Color[] colors = {
            Color.blue, 
            Color.red, 
            new Color(0, 180, 0) /* darker green than Color.green */, 
            Color.magenta, 
            Color.cyan, 
            Color.orange, 
            Color.pink};
    
    public static final int maxColors = colors.length; 
    
    public ColoredRenderer() {
        super();
    }
    
    public Component getTableCellRendererComponent(JTable table, Object value,
            boolean isSelected, boolean hasFocus, int row, int column) {
        if (!isSelected) {
            int c = ((Double) table.getValueAt(row, table.convertColumnIndexToView(table.getColumnCount() - 1))).intValue() - 1;
            super.setForeground(colors[c % colors.length]);
        }
        
        this.setAlternatingBackground(this, table, isSelected, hasFocus, row, column);
        
        if(table.convertColumnIndexToModel(column) == table.getColumnCount() - 1) {
            //make the classification column blend in
            setForeground(this.getBackground());
            setValue("");
        } else        
            setValue(value); 
        
        return this;
    }    
}
