package edu.duke.cs.smlr.gui.table;

import javax.swing.table.AbstractTableModel;

/**A table model that prohibits any editing of cells.
 * 
 * @author Matt Edwards
 */

public class UneditableModel extends AbstractTableModel {

    protected String[] columnNames;
    protected Object[][] values;
    
    public UneditableModel(Object[][] a, String b[]) {
        values = a;
        columnNames = b;
    }
    
    /* (non-Javadoc)
     * @see javax.swing.table.TableModel#getRowCount()
     */
    public int getRowCount() {
        return values.length;
    }

    /* (non-Javadoc)
     * @see javax.swing.table.TableModel#getColumnCount()
     */
    public int getColumnCount() {
        return columnNames.length;
    }

    /* (non-Javadoc)
     * @see javax.swing.table.TableModel#getValueAt(int, int)
     */
    public Object getValueAt(int rowIndex, int columnIndex) {
        return values[rowIndex][columnIndex];
    }
    
    public String getColumnName(int col) {
        return columnNames[col];
    }
    
    public Class getColumnClass(int column) {
        return getValueAt(0, column).getClass();
    }
    
    public void setValueAt(Object value, int row, int col) {
        values[row][col] = value;
        fireTableCellUpdated(row, col);
    }    

}