package edu.duke.cs.smlr.learn;

import java.io.BufferedReader;
import java.io.IOException;

import edu.duke.cs.smlr.factories.MatrixFactory;
import edu.duke.cs.smlr.learn.interfaces.IMatrix;
import edu.duke.cs.smlr.util.HaltingException;

/**
 * Reads a classifier from a specific classifier file. This option is not included in the current release (1.0.1)
 * 
 * @author Jason Bosko, Matt Edwards
 *
 */
public class ClassifierReader {
	
	private ClassifierReader() {}
	
	public static IMatrix getClassifier(BufferedReader input) throws IOException {
		String line;
		IMatrix weights = null;
		int row = 0;
		
		while ((line = input.readLine()) != null) {
			if (line.startsWith("$")) {				
				weights = MatrixFactory.makeMatrix(
						Integer.parseInt(line.split(" $")[0]), 
						Integer.parseInt(line.split(" $")[1]));				
			}
			
			if (line.startsWith(">")) {
				try {					
					if (weights == null) throw new IndexOutOfBoundsException();
					String [] elements = line.split("> ");
					for (int i = 0; i < elements.length; ++i)
						weights.set(Double.parseDouble(elements[i]), row, i);
				} 
				catch (IndexOutOfBoundsException e) { 
					throw new HaltingException("Malformed classifier file!");
				}				
			}			
		}
		return weights;		
	}
}
