package edu.duke.cs.smlr.learn;


import java.util.ArrayList;
import java.util.Iterator;

import edu.duke.cs.smlr.learn.interfaces.IMatrix;
import edu.duke.cs.smlr.learn.properties.ModifyProperty;

/**
 * This static class performs modifications to the data (row /
 * column removal).  It originally normalized as well, but then we
 * realized that that is learning-theoretically not good.
 * 
 * @author Matt Edwards, Jason Bosko
 */

public class Modifier {
	/**
	 * Modifies the classes matrix according to the options in <b>property</b>.
	 * @param classes The classes matrix.
	 * @param property The property containing the options for modification.
	 * @return The modified matrix.
	 */
	public static IMatrix modifyClasses(IMatrix classes, ModifyProperty property) {
		ArrayList rows = new ArrayList();
		ArrayList cols = new ArrayList();

		for (int i = 0; i < classes.numRows(); ++i)
			rows.add(new Integer(i));
		for (int i = 0; i < classes.numCols(); ++i)
			cols.add(new Integer(i));

		for (Iterator it = property.getDeletedRows().iterator(); it.hasNext(); 
			rows.remove(it.next()));

		classes = classes.chunk(rows, cols);
		return classes;
	}

	/**
	 * Modifies the data matrix according to the options in <b>property</b>.
	 * @param classes The data matrix.
	 * @param property The property containing the options for modification.
	 * @return The modified matrix.
	 */
	public static IMatrix modifyData(IMatrix data, ModifyProperty property) {
		ArrayList rows = new ArrayList();
		ArrayList cols = new ArrayList();

		for (int i = 0; i < data.numRows(); ++i)
			rows.add(new Integer(i));
		for (int i = 0; i < data.numCols(); ++i)
			cols.add(new Integer(i));

		for (Iterator it = property.getDeletedRows().iterator(); it.hasNext(); rows
				.remove(it.next()))
			;
		for (Iterator it = property.getDeletedColumns().iterator(); it
				.hasNext(); cols.remove(it.next()))
			;

		data = data.chunk(rows, cols);

		return data;
	}
}
