package edu.duke.cs.smlr.learn;

import edu.duke.cs.smlr.factories.ClassifierFactory;
import edu.duke.cs.smlr.factories.KernelFactory;
import edu.duke.cs.smlr.learn.interfaces.IClassifier;
import edu.duke.cs.smlr.learn.interfaces.IKernel;
import edu.duke.cs.smlr.learn.interfaces.IMatrix;
import edu.duke.cs.smlr.learn.properties.TestProperty;
import edu.duke.cs.smlr.learn.results.TestResult;
import edu.duke.cs.smlr.util.DataPointMap;

/**
 * Class that applies a set of weights to a test data set. Converts a
 * TestProperty into a TestResult.
 * 
 * @author Matt Edwards, Jason Bosko
 */
public class Tester {
	private Tester () {} // no instantiation
	
	/**
	 * Tests a classifier based on the options specified in <b>settings</b>.
	 * @param settings The options for testing.
	 * @param testMap A map of data points in the test matrix to their actual location in the original data.
	 * @param trainMap A map of data points in the train matrix to their actual location in the original data.
	 * @param cache The kernel cache to be used. This should be <b>null</b> if no cache is needed/wanted.
	 * @return
	 */
	public static TestResult test(TestProperty settings, DataPointMap testMap, DataPointMap trainMap, KernelSource cache) {
		IMatrix kerneledData = null;//, kerneledData1 = null;
		if (cache != null) {
			kerneledData = cache.makeKernel(testMap, trainMap, settings.getBias());
		} 
		else 
		{
			IKernel kernel = KernelFactory.makeKernel(settings.getKernel());			
			kerneledData = kernel.computeKernel(settings.getTestData(),
					settings.getTrainingData(), settings.getKernelPar(), settings
					.getBias(), settings.isNormalize());
		}
/*
		if (kerneledData.sum() != kerneledData1.sum()) {
			System.out.println("Test data: " + settings.getTestData().numRows() + " by " + settings.getTestData().numCols());
			System.out.println("Training data: " + settings.getTrainingData().numRows() + " by " + settings.getTrainingData().numCols());
			System.out.println("Correct kerneled data: " + kerneledData.numRows() + " by " + kerneledData.numCols());
			System.out.println(kerneledData.chunk(0, 0, 3, 25));
			System.out.println("Incorrect kerneled data from cache: " + kerneledData1.numRows() + " by " + kerneledData1.numCols());			
			System.out.println(kerneledData1.chunk(0, 0, 3, 25));
			System.out.println(kerneledData.termDiv(kerneledData1).chunk(0, 0, 3, 25));
			"".charAt(4);
		}
		else
			System.err.println("Cached kernel evaluations in test are correct!");
		*/
		IClassifier classifier = ClassifierFactory.makeClassifier(0);
		
		TestResult result = new TestResult();
		result.setRealClassifications(settings.getClassifications());
		result.setClassifications(classifier.classifications(kerneledData,
				settings.getWeights()));
		result.setProbabilities(classifier.probabilities(kerneledData,
				settings.getWeights()));
		
		IMatrix classes = result.getClassifications();
		
		int errors = 0;
		for (int i = 0; i < classes.numRows(); ++i)
			if (classes.get(i, 0) != settings.getClassifications().get(i, 0)) {
				errors++;
				result.getMisclassifications().add(new Integer(i));
			}
		
		result.setNumCorrect(classes.numRows() - errors);
		result.setNumWrong(errors);
		return result;
	}
	
}
