package edu.duke.cs.smlr.learn.classifiers;

import edu.duke.cs.smlr.factories.MatrixFactory;
import edu.duke.cs.smlr.learn.interfaces.IClassifier;
import edu.duke.cs.smlr.learn.interfaces.IMatrix;

/**
 * Uses a multi-sigmoid function to calculate probabilities and other
 * information from a weight matrix and an input matrix
 * 
 * @author Jason Bosko, Matt Edwards
 * 
 */

public class DefaultClassifier implements IClassifier {

	// adapted from multiSigmoid by A. Hartemink and colleagues
	public IMatrix probabilities(IMatrix X, IMatrix w) {
		
		int N = X.numRows();//elements
		int M;				//classes
		int d = X.numCols();//features


		if ((w.numCols() == 1) || (w.numRows() == 1)) {
			int numelems = Math.max(w.numCols(), w.numRows());
			M = numelems / d;
			w = w.reshape(d, M);
			
		} 
//		else 
//			if (X.numCols() != w.numRows()) //silly hack because it was breaking here
//				w = w.transpose();
		
		M = w.numCols() + 1;
		
		IMatrix b = X.mult(w);
		for (int k = 0; k < N; ++k)
			for (int l = 0; l < M - 1; ++l)
				b.set(Math.min(b.get(k,l), 100), k, l);
		IMatrix a = MatrixFactory.makeMatrix(N, M); // will be N by M
		for (int k = 0; k < N; ++k)
			for (int l = 0; l < M - 1; ++l)
				a.set(b.get(k, l), k, l);
		for (int k = 0; k < N; k++) {
			a.set(0, k, M - 1);
		}

		IMatrix P = MatrixFactory.makeMatrix(N, M);
		IMatrix ones = MatrixFactory.makeMatrix(1, M); // 1 by M
		for (int k = 0; k < M; ++k)
			ones.set(1, 0, k);
		a.exp();
		P = a.termDiv(a.rowSum().mult(ones));
		return P;
	}

	public IMatrix maxProbabilities(IMatrix x, IMatrix w) {
		IMatrix temp = probabilities(x, w);
		return temp.maxEachRow();
	}

	public IMatrix classifications(IMatrix x, IMatrix w) {
		IMatrix temp = probabilities(x, w);
		return temp.maxIndices().add(1); // go from 0-based to 1-based
	}

}
