package edu.duke.cs.smlr.learn.interfaces;

/**
 * Interface for classifying inputs with weights
 * 
 * @author Jason Bosko, Matt Edwards
 */
public interface IClassifier {

	/**
	 * Computes probalities from input data matrix and a weight matrix
	 * 
	 * @param x
	 *            the input data, NxD
	 * @param w
	 *            the weights, DxM
	 * @return a matrix NxM representing the respective probabilities that an
	 *         input belongs to a class
	 */
	public IMatrix probabilities(IMatrix x, IMatrix w);

	/**
	 * Computes the highest probability of a classification for each element
	 * 
	 * @param x
	 *            the input data, NxD
	 * @param w
	 *            the weights, DxM
	 * @return a matrix (column vector) Nx1 representing the highest probability
	 *         of each element
	 */
	public IMatrix maxProbabilities(IMatrix x, IMatrix w);

	/**
	 * Classifies input data according to weights
	 * 
	 * @param x
	 *            the input data, NxD
	 * @param w
	 *            the weights, DxM
	 * @return a matrix (column vector) Nx1 representing the computed
	 *         classification (1 to M)
	 */
	public IMatrix classifications(IMatrix x, IMatrix w);
}
