package edu.duke.cs.smlr.learn.interfaces;

/**
 * Interface for kernels to perform on data
 * 
 * @author Jason Bosko, Matt Edwards
 */
public interface IKernel {

	/**
	 * Applies the kernel trick on input data
	 * 
	 * @param X1
	 *            the original input feature data, unaltered
	 * @param X2
	 *            the original input feature data, unaltered
	 * @param kernelPar
	 *            a parameter for the kernel (used if necessary)
	 * @param bias
	 *            The bias term to be added after kerneling
	 * @param normalize
	 * 			  whether the data should be normalized before kerneling
	 * @return the computed matrix
	 */
	public IMatrix computeKernel(IMatrix X1, IMatrix X2, Object kernelPar,
			double bias, boolean normalize);
}
