package edu.duke.cs.smlr.learn.interfaces;

import edu.duke.cs.smlr.learn.results.LearnResult;

/**
 * Interface for learning a set of weights to classify example data
 * 
 * @author Jason Bosko, Matt Edwards
 */

public interface ILearner {
	/**
	 * Learns weight functions given data, classifications, and various options
	 * 
	 * @param data
	 *            A Nxd matrix of features
	 * @param classifications
	 *            A NxM matrix of classifications that correspond to the
	 *            features
	 * @param lambda
	 *            An option for controlling sparsity
	 * @param convergenceTol
	 *            An option for controlling the convergence tolerance
	 * @param killThreshold
	 *            An option for controlling when a number is considered to be
	 *            zero
	 * @param startAt
	 *            An option to set the first guess for the weight functions
	 * @param method
	 *            Decides what algorithm to use (ML, Gaussian, or Laplacian)
	 * @param overrelaxed
	 *            Decides whether overrelaxed optimization is used
	 * @param cacheExp
	 * 			  Decides whether exponentials should be cached
	 * @param maxIterations
	 *            The maximum number of iterations allowed before the routine
	 *            exits
	 * @return A matrix of weight functions
	 */
	public LearnResult learnWeights(IMatrix data, IMatrix classifications,
			double lambda, double convergenceTol, double killThreshold,
			IMatrix startAt, int method, boolean overrelaxed, boolean cacheExp, 
			int maxIterations);
}
