package edu.duke.cs.smlr.learn.kernels;

import edu.duke.cs.smlr.learn.interfaces.IKernel;
import edu.duke.cs.smlr.learn.interfaces.IMatrix;

/**
 * Computes a cosine-based kernel
 * 
 * @author Jason Bosko, Matt Edwards
 * 
 */
public class CosineKernel extends AbstractKernel implements IKernel {
	
	public IMatrix computeKernel(IMatrix X1, IMatrix X2, Object useless,
			double bias, boolean normalize) {
		if (normalize) {
			X1 = normalizeTest(X1, X2);
			X2 = normalizeTrain(X2, X2);
		} else {
			X1 = X1.copy();
			X2 = X2.copy();
		}
		
		int x1Rows = X1.numRows();
		int x2Rows = X2.numRows();
		int x1Cols = X1.numCols();
		int x2Cols = X2.numCols();
		
		for (int i = 0; i < x1Rows; ++i) {
			double norm = X1.chunk(i, 0, i + 1, x1Cols).norm();
			for (int j = 0; j < x1Cols; ++j)
				X1.set(X1.get(i, j) / norm, i, j);
		}
		for (int i = 0; i < x2Rows; ++i) {
			double norm = X2.chunk(i, 0, i + 1, x2Cols).norm();
			for (int j = 0; j < x2Cols; ++j)
				X2.set(X2.get(i, j) / norm, i, j);
		}
		IMatrix temp = X1.mult(X2.copy().transpose());
		return addBias(temp, bias);
	}
	
}
