package edu.duke.cs.smlr.learn.kernels;

import edu.duke.cs.smlr.learn.interfaces.IKernel;
import edu.duke.cs.smlr.learn.interfaces.IMatrix;

/**
 * Computes a Radial Base Function (RBF) kernel
 * 
 * @author Matt Edwards, Jason Bosko
 * 
 */
public class RBFKernel extends AbstractKernel implements IKernel {
	
	public IMatrix computeKernel(IMatrix X1, IMatrix X2, Object kernelPar,
			double bias, boolean normalize) {
		
		if (normalize) {
			X1 = normalizeTest(X1, X2);
			X2 = normalizeTrain(X2, X2);
		} else {
			X1 = X1.copy();
			X2 = X2.copy();			
		}				
		
		int n1 = X1.numRows();
		int n2 = X2.numRows();
		int d = X1.numCols();
		
		double bandwidth = 1.0;
		
		if (kernelPar != null)
			try { 
				bandwidth = ((Double) kernelPar).doubleValue();
			} catch (ClassCastException e) { }
			
			IMatrix tempM = X1.mult(X2.copy().transpose()).mult(-2.0);
			IMatrix sumX1 = (X1.termPow(2.0)).rowSum();
			IMatrix sumX2 = (X2.termPow(2.0)).rowSum().copy().transpose();
			IMatrix K = sumX1.repmat(1, n2).add(sumX2.repmat(n1, 1));
			
			K.add(tempM);
			K.mult(-1.0 / ((double) d) / bandwidth);
			K.exp();
			
			return addBias(K, bias);
			
			
	}
}
