package edu.duke.cs.smlr.learn.learners;

import edu.duke.cs.smlr.factories.MatrixFactory;
import edu.duke.cs.smlr.learn.interfaces.ILearner;
import edu.duke.cs.smlr.learn.interfaces.IMatrix;
import edu.duke.cs.smlr.learn.results.LearnResult;

/**An abstract learner interface that provides a method used in several
 * learners.
 * @author Matt Edwards, Jason Bosko
 */

public abstract class AbstractLearner implements ILearner {

    /**This converts a column matrix into a binary matrix.  Example: <br>
     * [[1]<br>
     *  [2]<br>
     *  [3]]<br>
     * Will be converted to: <br>
     * [[1 0 0]<br>
     *  [0 1 0]<br>
     *  [0 0 1]]
     * @param y the column matrix, unaltered
     * @return the new binary matrix, or the passed matrix if it isn't a column matrix
     */
    public IMatrix processClasses(IMatrix y) {
        
        int n = y.numRows();
        int m = y.numCols();
        // convert from 1..M to 0010 type stuff
        if (m == 1) {
            m = (int) y.maxAbsRowSum();
            IMatrix y1 = MatrixFactory.makeMatrix(n, m);
            for (int i = 1; i <= m; i++)
                for (int k = 0; k < n; k++)
                    if (y.get(k, 0) == i)
                        y1.set(1, k, i - 1);
            y = y1;
        }		
		
        return y;
    }
    
    public abstract LearnResult learnWeights(IMatrix data, IMatrix classifications,
            double lambda, double convergenceTol, double killThreshold,
            IMatrix startAt, int method, boolean overrelaxed, boolean cacheExp, int maxIterations);

}
