package edu.duke.cs.smlr.learn.properties;

import edu.duke.cs.smlr.util.DataPointMap;

/**A class that contains both a learnProperty and a testProperty.  Used for k-fold operations
 * where a learn and test set must be paired together.
 * 
 * @author Matt Edwards, Jason Bosko
 *
 */
public class ComboProperty {
	private LearnProperty learnProperty;
	private TestProperty testProperty;
	private DataPointMap learnDataMap;
	private DataPointMap testDataMap;
	

	private boolean display = true;

	public ComboProperty(LearnProperty lp) {
		learnProperty = new LearnProperty(lp);
		testProperty = new TestProperty(lp);
	}

	public ComboProperty(LearnProperty lp, TestProperty tp) {
		learnProperty = new LearnProperty(lp);
		testProperty = new TestProperty(tp);
	}

	public ComboProperty(ComboProperty cp) {
		learnProperty = cp.getLearnProperty();
		testProperty = cp.getTestProperty();
	}

	public ComboProperty() {
		learnProperty = new LearnProperty();
		testProperty = new TestProperty();
	}

	public String toString() {
		if (!display)
			return "";
		return learnProperty.toString() + testProperty.toString();
	}

	/**
	 * @return Returns the learnProperty.
	 */
	public LearnProperty getLearnProperty() {
		return learnProperty;
	}

	/**
	 * @return Returns the testProperty.
	 */
	public TestProperty getTestProperty() {
		return testProperty;
	}
	
	/**
	 * @return Returns the learnDataMap
	 */
	public DataPointMap getLearnDataMap() {
		return learnDataMap;
	}
	
	/**
	 * @return Returns the testDataMap
	 */
	public DataPointMap getTestDataMap() {
		return testDataMap;
	}
	
	/**
	 * @param m The DataPointMap for the training set.
	 */
	public void setLearnDataMap(DataPointMap m) {
		learnDataMap = m;
	}
	
	/**
	 * @param m The DataPointMap for the testing set.
	 */
	public void setTestDataMap(DataPointMap m) {
		testDataMap = m;
	}
}
