package edu.duke.cs.smlr.learn.properties;

import edu.duke.cs.smlr.util.Constants;

/**
 * Class for storing relevant properties for K-fold testing
 * 
 * @author Matt Edwards, Jason Bosko
 */
public class KFoldProperty {
	private LearnProperty learnProperty;
	private int k;
	private int behavior;

	private boolean display = true;

	public KFoldProperty() {

	}

	/**
	 * @param learnProperty the LearnProperty that this KFoldProperty uses
	 * @param k the k for this KFoldProperty
	 * @param behavior the behavior for splitting test from training
	 */	
	public KFoldProperty(LearnProperty learnProperty, int k, int behavior) {
		this.learnProperty = learnProperty;
		this.k = k;
		this.behavior = behavior;
	}

	public String toString() {
		if (!display)
			return "";
		final String sep = ", ";
		return "K = " + k + sep	+ "behavior = " + Constants.toString(behavior);
	}

	/**
	 * @return Returns the display.
	 */
	public boolean isDisplay() {
		return display;
	}

	/**
	 * @param display
	 *            The display to set.
	 */
	public void setDisplay(boolean display) {
		this.display = display;
	}

	/**
	 * @return Returns the behavior.
	 */
	public int getBehavior() {
		return behavior;
	}

	/**
	 * @param behavior
	 *            The behavior to set.
	 */
	public void setBehavior(int behavior) {
		this.behavior = behavior;
	}

	/**
	 * @return Returns the k.
	 */
	public int getK() {
		return k;
	}

	/**
	 * @param k
	 *            The k to set.
	 */
	public void setK(int k) {
		this.k = k;
	}

	/**
	 * @return Returns the learnProperty.
	 */
	public LearnProperty getLearnProperty() {
		return learnProperty;
	}

	/**
	 * @param learnProperty
	 *            The learnProperty to set.
	 */
	public void setLearnProperty(LearnProperty learnProperty) {
		this.learnProperty = learnProperty;
	}
}
