package edu.duke.cs.smlr.learn.properties;

import edu.duke.cs.smlr.learn.interfaces.IMatrix;

/**
 * Encapsulates the settings needed to perform a machine learning test on some
 * set of data
 * 
 * @author Matt Edwards, Jason Bosko
 */
public class TestProperty {
	private IMatrix testData;
	private IMatrix trainingData;
	private IMatrix classifications;
	private String testingFileName;
	private IMatrix weights;
	private String weightFileName;
	private Object kernelPar;
	private int kernel;
	private double bias;
	private boolean normalize;
	
	private boolean display;

	public TestProperty() {
		display = true;
	}

	/**
	 * Constructs a new TestProperty from the fields contained in <b>t</b>.
	 * @param t The TestProperty to be copied.
	 */
	public TestProperty(TestProperty t) {
		testData = t.getTestData();
		trainingData = t.getTrainingData();
		classifications = t.getClassifications();
		testingFileName = t.getTestingFileName();
		weights = t.getWeights();
		weightFileName = t.getWeightFileName();
		kernelPar = t.getKernelPar();
		kernel = t.getKernel();
		bias = t.getBias();
		normalize = t.isNormalize();
	}

	/**
	 * Constructs a new TestProperty from the settings in a LearnProperty that carry over to a test property.
	 * @param l The LearnProperty from which to carry over settings.
	 */
	public TestProperty(LearnProperty l) {
		testData = l.getTrainingData();
		trainingData = l.getTrainingData();
		classifications = l.getClassifications();
		weights = null;
		kernelPar = l.getKernelPar();
		kernel = l.getKernel();
		bias = l.getBias();
		normalize = l.isNormalize();
	}

	public String toString() {
		if (!display)
			return "";
		String s = new String();
		s += "Testing File name: " + testingFileName;
		return s;
	}

	/**
	 * @return Returns the trainingData.
	 */
	public IMatrix getTrainingData() {
		return trainingData;
	}

	/**
	 * @param trainingData
	 *            The trainingData to set.
	 */
	public void setTrainingData(IMatrix trainingData) {
		this.trainingData = trainingData;
	}

	/**
	 * @return Returns the display.
	 */
	public boolean isDisplay() {
		return display;
	}

	/**
	 * @param display
	 *            The display to set.
	 */
	public void setDisplay(boolean display) {
		this.display = display;
	}

	/**
	 * @return Returns the classifications.
	 */
	public IMatrix getClassifications() {
		return classifications;
	}

	/**
	 * @return Returns the data.
	 */
	public IMatrix getTestData() {
		return testData;
	}

	/**
	 * @return Returns the kernel.
	 */
	public int getKernel() {
		return kernel;
	}

	/**
	 * @return Returns the kernelPar.
	 */
	public Object getKernelPar() {
		return kernelPar;
	}

	/**
	 * @return Returns the weights.
	 */
	public IMatrix getWeights() {
		return weights;
	}

	/**
	 * @param classifications
	 *            The classifications to set.
	 */
	public void setClassifications(IMatrix classifications) {
		this.classifications = classifications;
	}

	/**
	 * @param data
	 *            The data to set.
	 */
	public void setTestData(IMatrix data) {
		this.testData = data;
	}

	/**
	 * @param kernel
	 *            The kernel to set.
	 */
	public void setKernel(int kernel) {
		this.kernel = kernel;
	}

	/**
	 * @param kernelPar
	 *            The kernelPar to set.
	 */
	public void setKernelPar(Object kernelPar) {
		this.kernelPar = kernelPar;
	}

	/**
	 * @param weights
	 *            The weights to set.
	 */
	public void setWeights(IMatrix weights) {
		this.weights = weights;
	}

	/**
	 * @return Returns the addBias.
	 */
	public double getBias() {
		return bias;
	}

	/**
	 * @param addBias
	 *            The addBias to set.
	 */
	public void setBias(double bias) {
		this.bias = bias;
	}

	/**
	 * @return Returns the testingFileName.
	 */
	public String getTestingFileName() {
		return testingFileName;
	}

	/**
	 * @param testingFileName
	 *            The testingFileName to set.
	 */
	public void setTestingFileName(String testingFileName) {
		this.testingFileName = testingFileName;
	}

	/**
	 * @return Returns the weightFileName.
	 */
	public String getWeightFileName() {
		return weightFileName;
	}

	/**
	 * @param weightFileName
	 *            The weightFileName to set.
	 */
	public void setWeightFileName(String weightFileName) {
		this.weightFileName = weightFileName;
	}

	public boolean isNormalize() {
		return normalize;
	}

	public void setNormalize(boolean normalize) {
		this.normalize = normalize;
	}
}
