package edu.duke.cs.smlr.learn.results;

import edu.duke.cs.smlr.learn.interfaces.IMatrix;
import edu.duke.cs.smlr.util.ClassificationMap;

/**
 * Class that stores the results from a series of tests generated by
 * using a K-Fold validation
 * 
 * @author Jason Bosko, Matt Edwards
 */
public class KFoldResult {
	private int K;
	public int divFactor;
	private ClassificationMap classificationMap;
	
	private int totalCorrect;
	private int totalIncorrect;
	private double avgBasisFunc;
	private double totalTime;
	private IMatrix averageWeights;
	private int weightCounts[][];
	private int numOfClassWrong[];
	private int numInClass[];
	
//	public String toString() {
//		String s = "";
//		String sep = "\n";
//		
//		s += "Total time used: " + totalTime + sep;
//		s += "Average number of bases used: "+ avgBasisFunc + sep;
//		s += sep;
//		if (displayTotalCorrect)
//			s += "Total Correct: " + totalCorrect + sep;
//		if (displayTotalIncorrect)
//			s += "Total Incorrect: " + totalIncorrect + sep;
//		if (displayTotalPercentage)
//			s += "Total Percentage: " + ((double)(totalCorrect*100) / (double)(totalCorrect+totalIncorrect)) + sep;
//		for (int i = 0; i < numOfClassWrong.length; i++) {
//			s += "Overall, " + numOfClassWrong[i] + " ("+((double)numOfClassWrong[i]*100/(double)numInClass[i])+"%) of the data points whose true class label is " + 
//			  classificationMap.getOriginalClassification(new Double(i+1)) + " were misclassified as some other class."+sep;
//		}
//		if (displayAverageWeights)
//			s += "Average Weights: " + sep + averageWeights.toString() + sep;
//		if (displayWeightCounts) {
//			s += "Weight Counts: " + sep;
//			for (int i = 0; i < weightCounts.length; ++i) {
//				boolean isZero = true;
//				for (int j = 0; j < weightCounts[0].length; ++j)
//					if (weightCounts[i][j] != 0) isZero = false;
//				if (!isZero) {
//					s += "Row " + (i+1) + ": ";
//					for (int j = 0; j < weightCounts[0].length; ++j)
//						s += " " + weightCounts[i][j] + " ";
//					s += sep;
//				}
//			}
//		}
//		if (displayAverageWeightCounts) {
//			s += sep + "Percentage of tests where a weight was used: " + sep;
//			for (int i = 0; i < weightCounts.length; ++i) {
//				boolean isZero = true;
//				for (int j = 0; j < weightCounts[0].length; ++j)
//					if (weightCounts[i][j] != 0) isZero = false;
//				if (!isZero) {
//					s += "Row " + (i+1) + ": ";
//					for (int j = 0; j < weightCounts[0].length; ++j)
//						s += " " + ((double)weightCounts[i][j] * 100.0 / divFactor) + " ";
//					s += sep;
//				}
//			}
//		}
//		return s;	
//	}
	
	/**
	 * @return Returns the averageWeights.
	 */
	public IMatrix getAverageWeights() {
		return averageWeights;
	}
	/**
	 * @param averageWeights The averageWeights to set.
	 */
	public void setAverageWeights(IMatrix averageWeights) {
		this.averageWeights = averageWeights;
	}

	/**
	 * @return Returns the k.
	 */
	public int getK() {
		return K;
	}
	/**
	 * @param k The k to set.
	 */
	public void setK(int k) {
		K = k;
	}
	/**
	 * @return Returns the totalCorrect.
	 */
	public int getTotalCorrect() {
		return totalCorrect;
	}
	/**
	 * @param totalCorrect The totalCorrect to set.
	 */
	public void setTotalCorrect(int totalCorrect) {
		this.totalCorrect = totalCorrect;
	}
	/**
	 * @return Returns the totalIncorrect.
	 */
	public int getTotalIncorrect() {
		return totalIncorrect;
	}
	/**
	 * @param totalIncorrect The totalIncorrect to set.
	 */
	public void setTotalIncorrect(int totalIncorrect) {
		this.totalIncorrect = totalIncorrect;
	}
	/**
	 * @return Returns the weightCounts.
	 */
	public int[][] getWeightCounts() {
		return weightCounts;
	}
	/**
	 * @param weightCounts The weightCounts to set.
	 */
	public void setWeightCounts(int[][] weightCounts) {
		this.weightCounts = weightCounts;
	}
	
	public double[][] getAvgWeightCounts() {
		double ret[][] = new double[weightCounts.length][weightCounts[0].length];
		for (int i = 0; i < weightCounts.length; ++i) {
			for (int j = 0; j < weightCounts[0].length; ++j)
					ret[i][j] = ((double)weightCounts[i][j] * 100.0 / divFactor);
		}
		return ret;
	}
	
	/**
	 * @return Returns the displayTotalPercentage.
	 */

	public void setClassificationMap(ClassificationMap classificationMap) {
		this.classificationMap = classificationMap;
	}

	/**
	 * @return Returns the numOfClassWrong.
	 */
	public int[] getNumOfClassWrong() {
		return numOfClassWrong;
	}

	/**
	 * @param numOfClassWrong The numOfClassWrong to set.
	 */
	public void setNumOfClassWrong(int[] numOfClassWrong) {
		this.numOfClassWrong = numOfClassWrong;
	}
	
	public String getExcludedBases(double thresh) {
		String ret = "";
		double total = thresh * divFactor;
		int excluded = 0;
		for (int i = 0; i < weightCounts.length; i++) {
			boolean excludeMe = true;
			for (int q = 0; q < weightCounts[i].length; q++) {
				if (weightCounts[i][q] >= total) excludeMe = false;
			}
			
			if (excludeMe) {
				if (excluded != 0) ret+=", ";
				ret += ""+(i+1);
				excluded++;
			}
		}
		ret = "("+excluded+" excluded): "+ret;
		return ret;
	}

	public double getAvgBasisFunc() {
		return avgBasisFunc;
	}

	public void setAvgBasisFunc(double basisFunc) {
		this.avgBasisFunc = basisFunc;
	}

	public double getTotalTime() {
		return totalTime;
	}

	public void setTotalTime(double totalTime) {
		this.totalTime = totalTime;
	}

	public int[] getNumInClass() {
		return numInClass;
	}

	public void setNumInClass(int[] numInClass) {
		this.numInClass = new int[numInClass.length];
		System.arraycopy(numInClass, 0, this.numInClass, 0, numInClass.length);
	}
	
	public String getMisclassString() {
		String s = ""; String sep = "\n";
		for (int i = 0; i < numOfClassWrong.length; i++) {
			s += "Overall, " + numOfClassWrong[i] + " ("+((double)numOfClassWrong[i]*100/(double)numInClass[i])+"%) of the data points whose true class label is " + 
			  classificationMap.getOriginalClassification(new Double(i+1)) + " were misclassified as some other class."+sep;
		}
		return s;
	}
	
	public String getWeightCountString() {
		String s = ""; String sep = "\n";
		s += "Weight Counts: " + sep;
		for (int i = 0; i < weightCounts.length; ++i) {
			boolean isZero = true;
			for (int j = 0; j < weightCounts[0].length; ++j)
				if (weightCounts[i][j] != 0) isZero = false;
			if (!isZero) {
				s += "Row " + (i+1) + ": ";
				for (int j = 0; j < weightCounts[0].length; ++j)
					s += " " + weightCounts[i][j] + " ";
				s += sep;
			}
		}
		return s;
	}
	public String getAvgWeightCountString() {
		String s = ""; String sep = "\n";
		s += sep + "Percentage of tests where a weight was used: " + sep;
		for (int i = 0; i < weightCounts.length; ++i) {
			boolean isZero = true;
			for (int j = 0; j < weightCounts[0].length; ++j)
				if (weightCounts[i][j] != 0) isZero = false;
			if (!isZero) {
				s += "Row " + (i+1) + ": ";
				for (int j = 0; j < weightCounts[0].length; ++j)
					s += " " + ((double)weightCounts[i][j] * 100.0 / divFactor) + " ";
				s += sep;
			}
		}
		return s;
	}
	

}
