package edu.duke.cs.smlr.learn.results;

import java.util.ArrayList;
import java.util.Iterator;

import edu.duke.cs.smlr.learn.interfaces.IMatrix;
import edu.duke.cs.smlr.util.ClassificationMap;
import edu.duke.cs.smlr.util.DataPointMap;

/**
 * Class that stores the results from a single testing task
 * 
 * @author Matt Edwards, Jason Bosko
 */
public class TestResult {
	
	private ClassificationMap classificationMap;
	private DataPointMap trainMap;
	private DataPointMap testMap;

	private IMatrix probabilities;
	private IMatrix classifications;
	private IMatrix realClassifications;
	private int numWrong;
	private int numCorrect;
	private ArrayList misclassifications;
	private int numWrongOfClass[];
	private int numInClass[];
	
	private int confusion[][];


	public TestResult() {
		misclassifications = new ArrayList();
	}
	
//	private static String convertedString(IMatrix y, ClassificationMap c, DataPointMap m) {
//		String s = "";
//		for (int i = 0; i < y.numRows(); ++i) {
//			s += (m.getOriginalDataPoint(new Integer(i))+1) + ": " + c.getOriginalClassification(new Double(y.get(i, 0))) + "\n";
//		}
//		return s;
//	}
	
	public String getClassificationsString() {
		String s = "";
		for (int i = 0; i < classifications.numRows(); ++i) {
			s += (testMap.getOriginalDataPoint(new Integer(i))+1) + ": " + classificationMap.getOriginalClassification(new Double(classifications.get(i, 0))) + "\n";
		}
		return s;
	}
	
	public String getProbabilitiesString() {
		String s = "";
		for (int i = 0; i < probabilities.numRows(); ++i) {
			s += (testMap.getOriginalDataPoint(new Integer(i))+1) + " :";
			for (int k = 0; k < probabilities.numCols(); k++) {
				s += " Class " + classificationMap.getOriginalClassification(new Double(k+1)) + " = "+probabilities.get(i, k);
			}
			s += "\n";
		}
		return s;
	}
	
//	public String toString() {
//		String s = "";
//		String sep = "\n";
//		if (displayProbabilities)
//			s += "Probabilities: \n" + maxProbabilities.toString(testMap) + sep;
//		if (displayClassifications)
//			s += "Classifications: \n" + convertedString(classifications, classificationMap, testMap) + sep;
//		if (displayNumWrong)
//			s += "Number Incorrect: " + numWrong + sep;
//		if (displayNumCorrect)
//			s += "Number Correct: " + numCorrect + sep;
//		if (displayPercentage)
//			s += "Percentage Correct: "
//					+ ((double) numCorrect / (double) (numCorrect + numWrong) * 100)
//					+ " %" + sep;
//
//		
//		if (displayMisclassifications) {
//			int tot = classificationMap.getTotalClasses();
//			numInClass = new int[tot];
//			for (int i = 0; i < tot; i++) numInClass[i] = 0;
//			for (int i = 0; i < realClassifications.numRows(); i++)
//				numInClass[((int) realClassifications.get(i,0))-1]++;
//			numWrongOfClass = new int[classificationMap.getTotalClasses()];
//			for (int i = 0; i < numWrongOfClass.length; i++)
//				numWrongOfClass[i]=0;
//			if (!misclassifications.isEmpty())
//				s += "Misclassifications: \n";
//			Iterator it = misclassifications.iterator();
//			while (it.hasNext()) {
//				int index = ((Integer) it.next()).intValue();
//				numWrongOfClass[(int) realClassifications.get(index, 0) - 1]++;
//				s += "Data point " + (testMap.getOriginalDataPoint(new Integer(index))+1) + " was misclassified as a " + classificationMap.getOriginalClassification(new Double(classifications.get(index, 0))) +
//				     " but is really a " + classificationMap.getOriginalClassification(new Double(realClassifications.get(index, 0))) + sep;
//			}
//			for (int i = 0; i < numWrongOfClass.length; i++) {
//				s += "Overall, " + numWrongOfClass[i] + " ("+((double)numWrongOfClass[i]*100/(double)numInClass[i])+"%) of the data points whose true class label is " + 
//					  classificationMap.getOriginalClassification(new Double(i+1)) + " were misclassified as some other class."+sep;
//			}
//			confusion = new int[classificationMap.getTotalClasses()][classificationMap.getTotalClasses()];
//			for (int i = 0; i < classifications.numRows(); i++) {
//				confusion[((int)realClassifications.get(i,0))-1][((int)classifications.get(i,0))-1]++;
//			}
//			
//		}
//		
//		return s;
//	}

	/**
	 * @return Returns the realClassifications.
	 */
	public IMatrix getRealClassifications() {
		return realClassifications;
	}
	/**
	 * @param realClassifications The realClassifications to set.
	 */
	public void setRealClassifications(IMatrix realClassifications) {
		this.realClassifications = realClassifications;
	}
	
	/**
	 * @return Returns the classifications.
	 */
	public IMatrix getClassifications() {
		return classifications;
	}

	/**
	 * @param classifications
	 *            The classifications to set.
	 */
	public void setClassifications(IMatrix classifications) {
		this.classifications = classifications;
	}

	/**
	 * @return Returns the maxProbabilities.
	 */
	public IMatrix getProbabilities() {
		return probabilities;
	}

	/**
	 * @param maxProbabilities
	 *            The maxProbabilities to set.
	 */
	public void setProbabilities(IMatrix maxProbabilities) {
		this.probabilities = maxProbabilities;
	}

	/**
	 * @return Returns the numCorrect.
	 */
	public int getNumCorrect() {
		return numCorrect;
	}

	/**
	 * @param numCorrect
	 *            The numCorrect to set.
	 */
	public void setNumCorrect(int numCorrect) {
		this.numCorrect = numCorrect;
	}

	/**
	 * @return Returns the numWrong.
	 */
	public int getNumWrong() {
		return numWrong;
	}

	/**
	 * @param numWrong
	 *            The numWrong to set.
	 */
	public void setNumWrong(int numWrong) {
		this.numWrong = numWrong;
	}

	
	/**
	 * @return Returns the misclassifications.
	 */
	public ArrayList getMisclassifications() {
		return misclassifications;
	}

	public void setClassificationMap(ClassificationMap classificationMap) {
		this.classificationMap = classificationMap;
	}
	public void setTrainMap(DataPointMap trainMap) {
		this.trainMap = trainMap;
	}
	public void setTestMap(DataPointMap testMap) {
		this.testMap = testMap;
	}
	
	/**
	 * Gets the confusion matrix for the test
	 * @return The confusion matrix
	 */
	public int[][] getConfusionMatrix() {
		try {
			confusion = new int[classificationMap.getTotalClasses()][classificationMap.getTotalClasses()];
			for (int i = 0; i < classifications.numRows(); i++) {
				confusion[((int)realClassifications.get(i,0))-1][((int)classifications.get(i,0))-1]++;
			}
			return confusion;
		} catch (Exception e) {
			return new int[0][0];
		}
	}
	
	public String getMisclassificationsString() {
		if (realClassifications.get(0,0) == 0) return "";
		String s = "";
		String sep = "\n";
		int tot = classificationMap.getTotalClasses();
		numInClass = new int[tot];
		for (int i = 0; i < tot; i++) numInClass[i] = 0;
		for (int i = 0; i < realClassifications.numRows(); i++)
			numInClass[((int) realClassifications.get(i,0))-1]++;
		numWrongOfClass = new int[classificationMap.getTotalClasses()];
		for (int i = 0; i < numWrongOfClass.length; i++)
			numWrongOfClass[i]=0;
		if (!misclassifications.isEmpty())
			s += "Misclassifications:";
		Iterator it = misclassifications.iterator();
		while (it.hasNext()) {
			int index = ((Integer) it.next()).intValue();
			numWrongOfClass[(int) realClassifications.get(index, 0) - 1]++;
			s += sep + "Data point " + (testMap.getOriginalDataPoint(new Integer(index))+1) + " was misclassified as a " + classificationMap.getOriginalClassification(new Double(classifications.get(index, 0))) +
			     " but is really a " + classificationMap.getOriginalClassification(new Double(realClassifications.get(index, 0)));
		}
		for (int i = 0; i < numWrongOfClass.length; i++) {
			s += sep + "Overall, " + numWrongOfClass[i] + " ("+((double)numWrongOfClass[i]*100/(double)numInClass[i])+"%) of the data points whose true class label is " + 
				  classificationMap.getOriginalClassification(new Double(i+1)) + " were misclassified as some other class.";
		}
		return s;
	}

	public DataPointMap getTrainMap() {
		return trainMap;
	}
}
