package edu.duke.cs.smlr.util;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

/**
 * A class that stores two HashMaps, a map of classification names to their numerical 
 * value and vice-versa. (i.e., if in the data, the classifications are "Rep." 
 * and "Dem.", then there needs to be a map that containts the numerical 
 * representation, 1 and 2, of these classes.)
 *  
 * @author Jason Bosko, Matt Edwards
 */
public class ClassificationMap {
	
	/**
	 * The mapping of the original class names to the new values
	 */
	private HashMap oldToNew;
	
	/**
	 * The mapping of the new class values to the original names
	 */
	private HashMap newToOld;
	
	/**
	 * The default constructor
	 */
	public ClassificationMap() {
		oldToNew = new HashMap();
		newToOld = new HashMap();
	}
	
	/**
	 * Creates both maps from a list of the original classes and a list of the 
	 * corresponding new values. Both lists must be of the same length, and 
	 * corresponding class values should be in the same position.
	 * @param originalClasses The original names for the classes.
	 * @param newClasses The new values for the classes.
	 */
	public ClassificationMap(ArrayList originalClasses, ArrayList newClasses) {
		oldToNew = new HashMap();
		newToOld = new HashMap();
		if (originalClasses.size() != newClasses.size())
			return;
		Iterator originalIter = originalClasses.iterator();
		Iterator newIter = newClasses.iterator();
		while (originalIter.hasNext()) {
			Object oldVal = originalIter.next();
			Object newVal = newIter.next();
			oldToNew.put(oldVal, newVal);
			newToOld.put(newVal, oldVal);
		}
	}
	
	/**
	 * Stores the mapping of oldVal to newVal and vice-versa
	 * @param oldVal the original value for the classification
	 * @param newVal the new value for the classification
	 */
	public void put(Object oldVal, Object newVal) {
		oldToNew.put(oldVal, newVal);
		newToOld.put(newVal, oldVal);
	}
	
	/**
	 * Gets the new classifcation for the original value
	 * @param oldVal the original classification
	 * @return the new value 
	 */
	public Object getNewClassification(Object oldVal) {
		return oldToNew.get(oldVal);
	}
	
	/**
	 * Gets the original classifcation for the new value
	 * @param newVal the new value
	 * @return the original classification
	 */
	public Object getOriginalClassification(Object newVal) {
		return newToOld.get(newVal);
	}
	
	/**
	 * Gives access to the oldToNew HashMap
	 * @return the oldToNew HashMap
	 */
	public HashMap getOldToNew() {
		return oldToNew;
	}
	
	/**
	 * Gives access to the newToOld HashMap
	 * @return the newToOld HashMap
	 */
	public HashMap getNewToOld() {
		return newToOld;
	}
	
	/**
	 * Gets the number of classes possible in classification map.
	 * @return The number of possible classes.
	 */
	public int getTotalClasses() {
		return oldToNew.size();
	}
}
