package edu.duke.cs.smlr.util;

/**
 * Contains the constants used for the settings in the various learning classes
 * 
 * @author Matt Edwards, Jason Bosko
 */
public final class Constants {
	
	public static final String NO_FILE_LOADED = "No File Loaded";
	
	public static final int KERNEL_RBF = 10;
	public static final int KERNEL_DIRECT = 20;
	public static final int KERNEL_LINEAR = 30;
	public static final int KERNEL_POLYNOMIAL = 40;
	public static final int KERNEL_COSINE = 50;

	public static final int PRIOR_LAPLACIAN = 15;
	public static final int PRIOR_GAUSSIAN = 25;
	public static final int PRIOR_NONE = 35;

	public static final int ALGORITHM_COMPONENT = 17;
	public static final int ALGORITHM_NON_COMPONENT = 27;

	public static final int TEST_TRAINING_DATA = 12;
	public static final int TEST_TESTING_DATA = 22;
	public static final int TEST_KFOLD = 32;
	public static final int TEST_LOOCV = 42;
	public static final int TEST_UNLABELED = 52;
	public static final int TEST_NONE = 62;

	public static final int KFOLD_PURE_RANDOM = 100;
	public static final int KFOLD_RATIO_RANDOM = 103;
	public static final int KFOLD_NON_RANDOM = 105;

	public static final int READER_FIRST_COLUMN = -10;
	public static final int READER_LAST_COLUMN = -20;
	public static final int READER_GUESS = -2;
	public static final int READER_UNCLASSIFIED = -100;

	/**
	 * Converts a constant into a string that represents the constant.
	 * @param c
	 * @return
	 */
	public static String toString(int c) {
		switch (c) {
		case KERNEL_COSINE:
			return "cosine";
		case KERNEL_DIRECT:
			return "direct";
		case KERNEL_LINEAR:
			return "linear";
		case KERNEL_POLYNOMIAL:
			return "polynomial";
		case KERNEL_RBF:
			return "rbf";

		case PRIOR_LAPLACIAN:
			return "laplacian";
		case PRIOR_GAUSSIAN:
			return "gaussian";
		case PRIOR_NONE:
			return "none";

		case ALGORITHM_COMPONENT:
			return "component";
		case ALGORITHM_NON_COMPONENT:
			return "noncomponent";

		case TEST_TRAINING_DATA:
			return "testontraining";
		case TEST_TESTING_DATA:
			return "testontesting";
		case TEST_KFOLD:
			return "kfoldtest";
		case TEST_LOOCV:
			return "loocv";
		case TEST_UNLABELED:
			return "unlabeled";
		case TEST_NONE:
			return "none";

		case KFOLD_PURE_RANDOM:
			return "random";
		case KFOLD_RATIO_RANDOM:
			return "ratiorandom";
		case KFOLD_NON_RANDOM:
			return "inplace";

		case READER_FIRST_COLUMN:
			return "first";
		case READER_LAST_COLUMN:
			return "last";
		case READER_GUESS:
			return "guess";

		default:
			throw new IllegalArgumentException("Unrecognized constant");
		}
	}
}
