package edu.duke.cs.smlr.util;

import java.util.HashMap;
import java.util.Iterator;

/**
 * A class that stores two HashMaps, a map of data points (as Integers) 
 * in the original data file to their place in the training/test data matrices, 
 * and vice-versa. (i.e., if a piece of data is the 3rd point in the original data 
 * file, it may become the 20th point in a training set generated through k-folding.
 *  
 * @author Jason Bosko, Matt Edwards
 */
public class DataPointMap {
	/**
	 * The mapping of the original class names to the new values
	 */
	private HashMap oldToNew;
	
	/**
	 * The mapping of the new class values to the original names
	 */
	private HashMap newToOld;
	
	/**
	 * The default constructor
	 */
	public DataPointMap() {
		oldToNew = new HashMap();
		newToOld = new HashMap();
	}
	
	/**
	 * Stores the mapping of oldVal to newVal and vice-versa
	 * @param oldVal the original data point index
	 * @param newVal the new data point index
	 */
	public void put(Integer oldVal, Integer newVal) {
		oldToNew.put(oldVal, newVal);
		newToOld.put(newVal, oldVal);
	}
	
	/**
	 * Gets the new data point corresponding to the original value
	 * @param oldVal the original data point index
	 * @return the new value 
	 */
	public int getNewDataPoint(Integer oldVal) {
		return ((Integer) oldToNew.get(oldVal)).intValue();
	}
	
	/**
	 * Gets the original data point corresponding to the new value
	 * @param newVal the new data point
	 * @return the original data point
	 */
	public int getOriginalDataPoint(Integer newVal) {
		return ((Integer) newToOld.get(newVal)).intValue();
	}
	
	/**
	 * Gives access to the oldToNew HashMap
	 * @return the oldToNew HashMap
	 */
	public HashMap getOldToNew() {
		return oldToNew;
	}
	
	/**
	 * Gives access to the newToOld HashMap
	 * @return the newToOld HashMap
	 */
	public HashMap getNewToOld() {
		return newToOld;
	}
	
	public void printNewToOldMap() {
		for (Iterator it = newToOld.keySet().iterator();it.hasNext();) {
			Integer x = (Integer) it.next();
			System.out.println(x.intValue() + " --> " + ((Integer) newToOld.get(x)).intValue());
		}
	}
	
	/**
	 * Makes a DataPointMap that maps the points 0..(len-1) to themselves
	 * @param len The length of the mapping to create
	 * @return A mapping of the points 0..(len-1) to themselves
	 */
	public static DataPointMap makeSequentialMap(int len) {
		DataPointMap m = new DataPointMap();
		for (int i = 0; i < len; i++) m.put(new Integer(i), new Integer(i));
		return m;
	}
}
