package edu.duke.cs.smlr.util.matrix;

import edu.duke.cs.smlr.learn.interfaces.IMatrix;
import edu.duke.cs.smlr.util.DataPointMap;

/**A base matrix class that includes operations to share basic information about itself
 * and to make certain kinds of copies or new instances.  This is the first abstract
 * matrix class used in implementing the FullMatrix instance.
 *  
 * @author Matt Edwards
 */

public abstract class AbstractMatrix implements IMatrix {
    
    public AbstractMatrix() {
        this(0, 0);
    }
    
    private AbstractMatrix(int r, int c) {}
    
    public AbstractMatrix(IMatrix other) {
        this(other.numRows(), other.numCols());
        this.assign(other);
    }
    
    public abstract int numRows();

    public abstract int numCols();
    
    public abstract double getQuick(int r, int c);
    
    public abstract void setQuick(double d, int r, int c);
    
    public double get(int r, int c) throws IndexOutOfBoundsException {
        if(r < 0 || c < 0 || r >= numRows() || c >= numCols())
            throw new IndexOutOfBoundsException("[" + r + ", " + c + "]");
        return getQuick(r, c);
    }
    
    public void set(double d, int r, int c) throws IndexOutOfBoundsException {
        if(r < 0 || c < 0 || r >= numRows() || c >= numCols())
            throw new IndexOutOfBoundsException("[" + r + ", " + c + "]");
        setQuick(d, r, c);        
    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#clone()
     */
    public Object clone() {
        return copy();
    }
    
    public IMatrix copy() {
        IMatrix ans = like();
        for(int  i = 0; i < numRows(); ++i)
            for(int j = 0; j < numCols(); ++j)
                ans.set(get(i, j), i, j);
        return ans;       
    }
    public abstract IMatrix like(int r, int c);
    
    public IMatrix like() {
        return like(numRows(), numCols());
    }
    
    public void assign(IMatrix other) throws IllegalArgumentException {
        if(other.numRows() != numRows() || other.numCols() != numCols())
            throw new IllegalArgumentException("Matrix sizes do not match");        
        for(int i = 0; i < numRows(); ++i)
            for(int j = 0; j < numCols(); ++j)
                set(other.get(i, j), i, j);
    }
    
    public double[][] toArray() {
        double[][] ans = new double[numRows()][numCols()];
        for(int i = 0; i < numRows(); ++i)
            for(int j = 0; j < numCols(); ++j)
        		ans[i][j] = get(i, j);
        return ans;
    }
    
    public String toString() {
    	String ans = "";
    	String temp = "";
    	boolean add = false;
        for(int i = 0; i < numRows(); ++i) {
        	temp = "";            
        	add = false;
            for(int j = 0; j < numCols(); ++j) {
            	if (Math.abs(get(i, j)) > 0)
            		add = true;
            	temp += " " + Double.toString(get(i, j));
            }
            if (add)
            	ans += Integer.toString(i + 1) + " :" + temp + "\n";            
        }
        return ans;
    }
    
    public String toString(DataPointMap map) {
    	String ans = "";
    	String temp = "";
    	boolean add = false;
        for(int i = 0; i < numRows(); ++i) {
        	temp = "";            
        	add = false;
            for(int j = 0; j < numCols(); ++j) {
            	if (Math.abs(get(i, j)) > 0)
            		add = true;
            	temp += " " + Double.toString(get(i, j));
            }
            if (add)
            	ans += (map.getOriginalDataPoint(new Integer(i))+1) + " :" + temp + "\n";            
        }
        return ans;
    }
  
}
