package edu.duke.cs.smlr.util.matrix;

import cern.colt.matrix.DoubleMatrix2D;
import cern.colt.matrix.impl.DenseDoubleMatrix2D;
import cern.colt.matrix.linalg.Algebra;
import edu.duke.cs.smlr.learn.interfaces.IMatrix;

/**An implementation of the FullMatrix interface that uses the CERN Colt matrix
 * libraries (specifically, a DenseDouble2D matrix) as its backend.  It does not
 * override any methods.
 *  
 * @see DenseDoubleMatrix2D
 * 
 * @author Matt Edwards
 */

public class CernMatrix extends ManipMatrix
	implements IMatrix, Cloneable, java.io.Serializable {
    
    protected DoubleMatrix2D matrix;

    public CernMatrix() {
        super();
    }

    public CernMatrix(IMatrix other) {
        super(other);
    }
    
    public CernMatrix(int r, int c) {
        matrix = new DenseDoubleMatrix2D(r, c);
    }

    /* (non-Javadoc)
     * @see edu.duke.cs.smlr.util.matrix.MathMatrix#transpose()
     */
    public IMatrix transpose() {
        matrix = matrix.viewDice();
        return this;
    }

    /* (non-Javadoc)
     * @see edu.duke.cs.smlr.util.matrix.Matrix#numRows()
     */
    public int numRows() {
        return matrix.rows();
    }

    /* (non-Javadoc)
     * @see edu.duke.cs.smlr.util.matrix.Matrix#numCols()
     */
    public int numCols() {
        return matrix.columns();
    }

    /* (non-Javadoc)
     * @see edu.duke.cs.smlr.util.matrix.Matrix#getQuick(int, int)
     */
    public double getQuick(int r, int c) {
        return matrix.getQuick(r, c);
    }

    /* (non-Javadoc)
     * @see edu.duke.cs.smlr.util.matrix.Matrix#setQuick(double, int, int)
     */
    public void setQuick(double d, int r, int c) {
        matrix.setQuick(r, c, d);
    }

    /* (non-Javadoc)
     * @see edu.duke.cs.smlr.util.matrix.Matrix#like(int, int)
     */
    public IMatrix like(int r, int c) {
        return new CernMatrix(r, c);
    }
    
    /* (non-Javadoc)
     * @see edu.duke.cs.smlr.util.matrix.MathMatrix#inverse()
     */
    public IMatrix inverse() {
        matrix.assign(Algebra.DEFAULT.inverse(matrix));
        return this;
    }

}
