package edu.duke.cs.smlr.util.matrix;

import edu.duke.cs.smlr.learn.interfaces.IMatrix;

/**A matrix class that supports many linear-algebraic computations.  This is the
 * third abstract class used to implement the FullMatrix interface.
 * 
 * @author Matt Edwards
 */

public abstract class MathMatrix extends SimpleMatrix implements IMatrix {
    protected int iDF = 0;
    
    public MathMatrix() {
        super();
    }
    
    public MathMatrix(IMatrix other) {
        super(other);
    }
    
    private MathMatrix(int r, int c) {
    }
    
    public abstract IMatrix inverse() throws UnsupportedOperationException;

    public abstract IMatrix transpose();
    
    public IMatrix mult(IMatrix other) throws IllegalArgumentException {
        if(numCols() != other.numRows())
            throw new IllegalArgumentException("Dimension error in matrix multiplication");        
        IMatrix ans = like(numRows(), other.numCols());        
        double tempSum;        
        for(int i = 0; i < ans.numRows(); ++i)
            for(int j = 0; j < ans.numCols(); ++j) {
                tempSum = 0;
                for(int  k = 0; k < numCols(); ++k)
                    tempSum += getQuick(i, k) * other.getQuick(k, j);
                ans.setQuick(tempSum, i, j);
            }        
        return ans;
    }
    
    public IMatrix add(IMatrix other) throws IllegalArgumentException {
        if(numRows() != other.numRows() || numCols() != other.numCols())
            throw new IllegalArgumentException("Dimension error in matrix addition, " + numRows() + " != " + other.numRows() + " or " + numCols() + " != " + other.numCols());
        for(int i = 0; i < numRows(); ++i)
            for(int j = 0; j < numCols(); ++j)
                setQuick(getQuick(i, j) + other.getQuick(i, j), i, j);
        return this;
    }
    
    public IMatrix termMult(IMatrix other) throws IllegalArgumentException {
        if(numRows() != other.numRows() || numCols() != other.numCols())
            throw new IllegalArgumentException("Dimension error in matrix addition");
        for(int i = 0; i < numRows(); ++i)
            for(int j = 0; j < numCols(); ++j)
                setQuick(getQuick(i, j) * other.getQuick(i, j), i, j);
        return this;
    }
    
    public IMatrix termDiv(IMatrix other) throws IllegalArgumentException {
        if(numRows() != other.numRows() || numCols() != other.numCols())
            throw new IllegalArgumentException("Dimension error in matrix addition");
        for(int i = 0; i < numRows(); ++i)
            for(int j = 0; j < numCols(); ++j)
                setQuick(getQuick(i, j) / other.getQuick(i, j), i, j);
        return this;
    }    
    
    public IMatrix kronecker(IMatrix other) {
        IMatrix ans = like(numRows() * other.numRows(), numCols() * other.numCols());
        for (int i = 0; i < ans.numRows(); ++i)
            for (int j = 0; j < ans.numCols(); ++j)
                ans.setQuick(getQuick(i / other.numRows(), j / other.numCols())
                        * other.getQuick(i % other.numRows(), j % other.numCols()), i, j);
        return ans;
    }
    
    
}
